@@ -128,6 +128,10 @@
 #endif
 
 #if HAVE_SYS_POLL_H
+# if linux
+/* Linux is just soooo broken */
+#  define _GNU_SOURCE 1
+# endif
 # include <sys/poll.h>
 # if linux && !defined(POLLRDNORM)
 /* Linux 2.1.xx supports poll(), header files are not upto date yet */
@@ -218,7 +222,7 @@
 #endif
 
 #if defined(INET6) && defined(CLIENT_COMPILE)
-# if (defined(linux) || defined(__NetBSD__) || defined(__osf__)) && \
+# if (defined(linux) || defined(__NetBSD__) || defined(__FreeBSD__) || defined(__osf__)) && \
 	HAVE_RESOLV_H
 #  include <resolv.h>
 # endif
@@ -227,6 +231,16 @@
 # endif
 #endif
 
+#if defined(HAVE_DLFCN_H)
+# include <dlfcn.h>
+#endif
+
+#if (defined(__FreeBSD__) ||\
+     defined(__OpenBSD__) ||\
+     defined(__NetBSD__)     ) && !defined(__ELF__)
+# define DLSYM_NEEDS_UNDERSCORE
+#endif
+
 /*  Some special include files for a special OS. :)
  */
 
@@ -470,7 +484,11 @@ extern char *inet_ntoa __P((struct in_addr in));
 #  include <cursesX.h>
 # endif
 # if (USE_NCURSES || USE_CURSES) && HAVE_CURSES_H
-#  include <curses.h>
+#  if HAVE_NCURSES_H
+#   include <ncurses.h>
+#  else
+#   include <curses.h>
+#  endif
 # endif
 #else
 # undef DOCURSES
@@ -704,7 +722,7 @@ static unsigned char minus_one[]={ 255, 255, 255, 255, 255, 255, 255, 255, 255,
 
 # define	AFINET		AF_INET6
 # define	SOCKADDR_IN	sockaddr_in6
-# define	SOCKADDR	sockaddr_in6
+# define	SOCKADDR	sockaddr
 # define	SIN_FAMILY	sin6_family
 # define	SIN_PORT	sin6_port
 # define	SIN_ADDR	sin6_addr
@@ -719,7 +737,7 @@ static unsigned char minus_one[]={ 255, 255, 255, 255, 255, 255, 255, 255, 255,
 char mydummy[MYDUMMY_SIZE];
 char mydummy2[MYDUMMY_SIZE];
 
-# if defined(linux) || defined(__NetBSD__)
+# if defined(linux) || defined(__NetBSD__) || defined(__FreeBSD__) || defined(bsdi)
 #  ifndef s6_laddr
 #   define s6_laddr        s6_addr32
 #  endif
