@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: send.c,v 1.30 1998/12/28 15:44:57 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: send.c,v 1.39.2.13 2001/10/18 21:43:05 chopin Exp $";
 #endif
 
 #include "os.h"
@@ -86,6 +86,27 @@ char	*notice;
 }
 
 #ifndef CLIENT_COMPILE
+/*
+** flush_fdary
+**      Used to empty all output buffers for connections in fdary.
+*/
+void    flush_fdary(fdp)
+FdAry   *fdp;
+{
+        int     i;
+        aClient *cptr;
+
+        for (i = 0; i <= fdp->highest; i++)
+            {
+                if (!(cptr = local[fdp->fd[i]]))
+                        continue;
+                if (!IsRegistered(cptr)) /* is this needed?? -kalt */
+                        continue;
+                if (DBufLength(&cptr->sendQ) > 0)
+                        (void)send_queued(cptr);
+            }
+}
+
 /*
 ** flush_connections
 **	Used to empty all output buffers for all connections. Should only
@@ -98,7 +119,6 @@ char	*notice;
 void	flush_connections(fd)
 int	fd;
 {
-#ifdef SENDQ_ALWAYS
 	Reg	int	i;
 	Reg	aClient *cptr;
 
@@ -110,7 +130,6 @@ int	fd;
 	    }
 	else if (fd >= 0 && (cptr = local[fd]) && DBufLength(&cptr->sendQ) > 0)
 		(void)send_queued(cptr);
-#endif
 }
 #endif
 
@@ -119,7 +138,6 @@ int	fd;
 **	Internal utility which delivers one message buffer to the
 **	socket. Takes care of the error handling and buffering, if
 **	needed.
-**	if SENDQ_ALWAYS is defined, the message will be queued.
 **	if ZIP_LINKS is defined, the message will eventually be compressed,
 **	anything stored in the sendQ is compressed.
 */
@@ -127,7 +145,7 @@ static	int	send_message(to, msg, len)
 aClient	*to;
 char	*msg;	/* if msg is a null pointer, we are flushing connection */
 int	len;
-#ifdef SENDQ_ALWAYS
+#if !defined(CLIENT_COMPILE)
 {
 	int i;
 
@@ -138,24 +156,21 @@ int	len;
 	if (to->fd < 0)
 	    {
 		Debug((DEBUG_ERROR,
-		      "Local socket %s with negative fd... AARGH!",
+		       "Local socket %s with negative fd... AARGH!",
 		      to->name));
 	    }
-#ifndef	CLIENT_COMPILE
-	else if (IsMe(to))
+	if (IsMe(to))
 	    {
 		sendto_flag(SCH_ERROR, "Trying to send to myself! [%s]", msg);
 		return 0;
 	    }
-#endif
 	if (IsDead(to))
 		return 0; /* This socket has already been marked as dead */
-# ifndef	CLIENT_COMPILE
 	if (DBufLength(&to->sendQ) > get_sendq(to))
-	    {
-#  ifdef HUB
+	{
+# ifdef HUB
 		if (CBurst(to))
-		    {
+		{
 			aConfItem	*aconf = to->serv->nline;
 
 			poolsize -= MaxSendq(aconf->class) >> 1;
@@ -165,65 +180,62 @@ int	len;
 				    "New poolsize %d. (sendq adjusted)",
 				    poolsize);
 			istat.is_dbufmore++;
-		    }
-		else if (IsServer(to) || IsService(to))
-			sendto_flag(SCH_ERROR,
-				"Max SendQ limit exceeded for %s: %d > %d",
-			   	get_client_name(to, FALSE),
-				DBufLength(&to->sendQ), get_sendq(to));
-		if (!CBurst(to))
-		    {
-			to->exitc = EXITC_SENDQ;
-			return dead_link(to, "Max Sendq exceeded");
-		    }
-#  else /* HUB */
-		if (IsService(to) || IsServer(to))
-			sendto_flag(SCH_ERROR,
-				"Max SendQ limit exceeded for %s: %d > %d",
-			   	get_client_name(to, FALSE),
-				DBufLength(&to->sendQ), get_sendq(to));
-		to->exitc = EXITC_SENDQ;
-		return dead_link(to, "Max Sendq exceeded");
-#  endif /* HUB */
-	    }
-	else
+		}
+		else
 # endif
-	    {
-tryagain:
+		{
+			char ebuf[BUFSIZE];
+
+			ebuf[0] = '\0';
+			if (IsService(to) || IsServer(to))
+			{
+				SPRINTF(ebuf,
+				"Max SendQ limit exceeded for %s: %d > %d",
+					get_client_name(to, FALSE),
+					DBufLength(&to->sendQ), get_sendq(to));
+			}
+			to->exitc = EXITC_SENDQ;
+			return dead_link(to, ebuf[0] ? ebuf :
+				"Max Sendq exceeded");
+		}
+	}
 # ifdef	ZIP_LINKS
-	        /*
-		** data is first stored in to->zip->outbuf until
-		** it's big enough to be compressed and stored in the sendq.
-		** send_queued is then responsible to never let the sendQ
-		** be empty and to->zip->outbuf not empty.
-		*/
-		if (to->flags & FLAGS_ZIP)
-			msg = zip_buffer(to, msg, &len, 0);
+	/*
+	** data is first stored in to->zip->outbuf until
+	** it's big enough to be compressed and stored in the sendq.
+	** send_queued is then responsible to never let the sendQ
+	** be empty and to->zip->outbuf not empty.
+	*/
+	if (to->flags & FLAGS_ZIP)
+		msg = zip_buffer(to, msg, &len, 0);
 
-		if (len && (i = dbuf_put(&to->sendQ, msg, len)) < 0)
+tryagain:
+	if (len && (i = dbuf_put(&to->sendQ, msg, len)) < 0)
 # else 	/* ZIP_LINKS */
-		if ((i = dbuf_put(&to->sendQ, msg, len)) < 0)
+tryagain:
+	if ((i = dbuf_put(&to->sendQ, msg, len)) < 0)
 # endif	/* ZIP_LINKS */
-			if (i == -2 && CBurst(to))
-			    {	/* poolsize was exceeded while connect burst */
-				aConfItem	*aconf = to->serv->nline;
+	{
+		if (i == -2 && CBurst(to))
+		    {	/* poolsize was exceeded while connect burst */
+			aConfItem	*aconf = to->serv->nline;
 
-				poolsize -= MaxSendq(aconf->class) >> 1;
-				IncSendq(aconf->class);
-				poolsize += MaxSendq(aconf->class) >> 1;
-				sendto_flag(SCH_NOTICE,
-					    "New poolsize %d. (reached)",
-					    poolsize);
-				istat.is_dbufmore++;
-				goto tryagain;
-			    }
-			else
-			    {
-				to->exitc = EXITC_MBUF;
-				return dead_link(to,
-					"Buffer allocation error for %s");
-			    }
-	    }
+			poolsize -= MaxSendq(aconf->class) >> 1;
+			IncSendq(aconf->class);
+			poolsize += MaxSendq(aconf->class) >> 1;
+			sendto_flag(SCH_NOTICE,
+				    "New poolsize %d. (reached)",
+				    poolsize);
+			istat.is_dbufmore++;
+			goto tryagain;
+		    }
+		else
+		    {
+			to->exitc = EXITC_MBUF;
+			return dead_link(to,
+				"Buffer allocation error for %s");
+		    }
+	}
 	/*
 	** Update statistics. The following is slightly incorrect
 	** because it counts messages even if queued, but bytes
@@ -245,7 +257,7 @@ tryagain:
 		send_queued(to);
 	return 0;
 }
-#else /* SENDQ_ALWAYS */
+#else /* CLIENT_COMPILE */
 {
 	int	rlen = 0, i;
 
@@ -259,97 +271,11 @@ tryagain:
 		      "Local socket %s with negative fd... AARGH!",
 		      to->name));
 	    }
-#ifndef	CLIENT_COMPILE
-	else if (IsMe(to))
-	    {
-		sendto_flag(SCH_ERROR, "Trying to send to myself! [%s]", msg);
-		return 0;
-	    }
-#endif
 	if (IsDead(to))
 		return 0; /* This socket has already been marked as dead */
 
-	/*
-	** DeliverIt can be called only if SendQ is empty...
-	*/
-	if ((DBufLength(&to->sendQ) == 0) &&
-	    (rlen = deliver_it(to, msg, len)) < 0)
+	if ((rlen = deliver_it(to, msg, len)) < 0 && rlen < len)
 		return dead_link(to,"Write error to %s, closing link");
-	else if (rlen < len)
-	    {
-		/*
-		** Was unable to transfer all of the requested data. Queue
-		** up the remainder for some later time...
-		*/
-# ifndef	CLIENT_COMPILE
-		if (DBufLength(&to->sendQ) > get_sendq(to))
-		    {
-#  ifdef HUB
-			if ((IsService(to) || IsServer(to)) && CBurst(to))
-			    {
-				aClass	*cl = to->serv->nline->class;
-
-				poolsize -= MaxSendq(cl) >> 1;
-				IncSendq(cl);
-				poolsize += MaxSendq(cl) >> 1;
-				sendto_flag(SCH_NOTICE,
-				    "New poolsize %d. (sendq adjusted)",
-                                            poolsize);
-				istat.is_dbufmore++;
-			    }
-			else if (IsServer(to) || IsService(to))
-#  else
-			if (IsServer(to) || IsService(to))
-#  endif
-			sendto_flag(SCH_ERROR,
-				"Max SendQ limit exceeded for %s: %d > %d",
-				 get_client_name(to, FALSE),
-				 DBufLength(&to->sendQ), get_sendq(to));
-			return dead_link(to, "Max Sendq exceeded");
-		    }
-		else
-# endif
-		    {
-tryagain:
-# ifdef	ZIP_LINKS
-	        /*
-		** data is first stored in to->zip->outbuf until
-		** it's big enough to be compressed and stored in the sendq.
-		** send_queued is then responsible to never let the sendQ
-		** be empty and to->zip->outbuf not empty.
-		*/
-			if (to->flags & FLAGS_ZIP)
-				msg = zip_buffer(to, msg, &len, 0);
-
-			if (len && (i = dbuf_put(&to->sendQ, msg+rlen,
-						 len-rlen)) < 0)
-# else 	/* ZIP_LINKS */
-			if ((i = dbuf_put(&to->sendQ, msg+rlen, len-rlen)) < 0)
-# endif	/* ZIP_LINKS */
-				if (i == -2 && CBurst(to))
-				    {
-				/* poolsize was exceeded while connect burst */
-					aConfItem *aconf = to->serv->nline;
-
-					poolsize -= MaxSendq(aconf->class) >>1;
-					IncSendq(aconf->class);
-					poolsize += MaxSendq(aconf->class) >>1;
-#ifndef	CLIENT_COMPILE
-					sendto_flag(SCH_NOTICE,
-					    "New poolsize %d. (reached)",
-						    poolsize);
-					istat.is_dbufmore++;
-#endif
-					goto tryagain;
-				    }
-				else
-				    {
-					to->exitc = EXITC_MBUF;
-					return dead_link(to,
-					 "Buffer allocation error for %s");
-				    }
-		    }
-	    }
 	/*
 	** Update statistics. The following is slightly incorrect
 	** because it counts messages even if queued, but bytes
@@ -386,11 +312,7 @@ aClient *to;
 		** not working correct if send_queued is called for a
 		** dead socket... --msa
 		*/
-#ifndef SENDQ_ALWAYS
-		return dead_link(to, "send_queued called for a DEADSOCKET:%s");
-#else
 		return -1;
-#endif
 	    }
 #ifdef	ZIP_LINKS
 	/*
@@ -458,7 +380,7 @@ aClient *to;
 
 
 #ifndef CLIENT_COMPILE
-static	anUser	ausr = { NULL, NULL, NULL, NULL, 0, 0, 0, 0, NULL, NULL, NULL,
+static	anUser	ausr = { NULL, NULL, NULL, NULL, 0, 0, 0, 0, NULL,
 			 NULL, "anonymous", "anonymous.", "anonymous."};
 
 static	aClient	anon = { NULL, NULL, NULL, &ausr, NULL, NULL, 0, 0,/*flags*/
@@ -678,6 +600,12 @@ int	sendto_one(aClient *to, char *pattern, ...)
 }
 #endif
 
+/*
+ * sendto_channel_butone
+ *
+ * Send a message to all members of a channel that are connected to this
+ * server except client 'one'.
+ */
 #ifndef CLIENT_COMPILE
 #if ! USE_STDARG
 /*VARARGS*/
@@ -705,7 +633,7 @@ void	sendto_channel_butone(aClient *one, aClient *from, aChannel *chptr, char *p
 
 	if (one != from && MyConnect(from) && IsRegisteredUser(from))
 	    {
-		/* useless junk? */
+		/* useless junk? */ /* who said that and why? --B. */
 #if ! USE_STDARG
 		sendto_prefix_one(from, from, pattern, p1, p2, p3, p4,
 				  p5, p6, p7, p8, p9, p10, p11);
@@ -797,76 +725,89 @@ void	sendto_serv_butone(aClient *one, char *pattern, ...)
 }
 
 #if ! USE_STDARG
-void	sendto_serv_v(one, ver, pattern, p1, p2, p3, p4,p5,p6,p7,p8,p9,p10,p11)
+int
+sendto_serv_v(one, ver, pattern, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11)
 aClient *one;
 int	ver;
 char	*pattern, *p1, *p2, *p3, *p4, *p5, *p6, *p7, *p8, *p9, *p10, *p11;
 #else
-void	sendto_serv_v(aClient *one, int ver, char *pattern, ...)
+int
+sendto_serv_v(aClient *one, int ver, char *pattern, ...)
 #endif
 {
-	Reg	int	i, len=0;
+	Reg	int	i, len=0, rc=0;
 	Reg	aClient *cptr;
 
 	for (i = fdas.highest; i >= 0; i--)
 		if ((cptr = local[fdas.fd[i]]) &&
-		    (!one || cptr != one->from) && !IsMe(cptr) &&
-		    (cptr->serv->version & ver)) {
-			if (!len)
+		    (!one || cptr != one->from) && !IsMe(cptr))
+			if (cptr->serv->version & ver)
 			    {
+				if (!len)
+				    {
 #if ! USE_STDARG
-				len = sendprep(pattern, p1, p2, p3, p4, p5,
-					       p6, p7, p8, p9, p10, p11);
+					len = sendprep(pattern, p1, p2, p3, p4,
+						       p5, p6, p7, p8, p9, p10,
+						       p11);
 #else
-				va_list	va;
-				va_start(va, pattern);
-				len = vsendprep(pattern, va);
-				va_end(va);
+					va_list	va;
+					va_start(va, pattern);
+					len = vsendprep(pattern, va);
+					va_end(va);
 #endif
+				    }
+				(void)send_message(cptr, sendbuf, len);
 			    }
-			(void)send_message(cptr, sendbuf, len);
-	}
-	return;
+			else
+				rc = 1;
+	return rc;
 }
 
 #if ! USE_STDARG
-void	sendto_serv_notv(one, ver, pattern, p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11)
+int
+sendto_serv_notv(one, ver, pattern, p1, p2, p3, p4, p5, p6, p7, p8, p9,p10,p11)
 aClient *one;
 int	ver;
 char	*pattern, *p1, *p2, *p3, *p4, *p5, *p6, *p7, *p8, *p9, *p10, *p11;
 #else
-void	sendto_serv_notv(aClient *one, int ver, char *pattern, ...)
+int
+sendto_serv_notv(aClient *one, int ver, char *pattern, ...)
 #endif
 {
-	Reg	int	i, len=0;
+	Reg	int	i, len=0, rc=0;
 	Reg	aClient *cptr;
 
 	for (i = fdas.highest; i >= 0; i--)
 		if ((cptr = local[fdas.fd[i]]) &&
-		    (!one || cptr != one->from) && !IsMe(cptr) &&
-		    ((cptr->serv->version & ver) == 0)) {
-			if (!len)
+		    (!one || cptr != one->from) && !IsMe(cptr))
+			if ((cptr->serv->version & ver) == 0)
 			    {
+				if (!len)
+				    {
 #if ! USE_STDARG
-				len = sendprep(pattern, p1, p2, p3, p4, p5,
-					       p6, p7, p8, p9, p10, p11);
+					len = sendprep(pattern, p1, p2, p3, p4,
+						       p5, p6, p7, p8, p9, p10,
+						       p11);
 #else
-				va_list	va;
-				va_start(va, pattern);
-				len = vsendprep(pattern, va);
-				va_end(va);
+					va_list	va;
+					va_start(va, pattern);
+					len = vsendprep(pattern, va);
+					va_end(va);
 #endif
+				    }
+				(void)send_message(cptr, sendbuf, len);
 			    }
-			(void)send_message(cptr, sendbuf, len);
-	}
-	return;
+			else
+				rc = 1;
+	return rc;
 }
 
 /*
  * sendto_common_channels()
  *
  * Sends a message to all people (inclusing user) on local server who are
- * in same channel with user.
+ * in same channel with user, except for channels set Quiet or Anonymous
+ * The calling procedure must take the necessary steps for such channels.
  */
 #if ! USE_STDARG
 /*VARARGS*/
@@ -913,8 +854,12 @@ void	sendto_common_channels(aClient *user, char *pattern, ...)
 			    user == cptr || !user->user)
 				continue;
 			for (lp = user->user->channel; lp; lp = lp->next)
-				if (IsMember(cptr, lp->value.chptr) &&
-				    !IsQuiet(lp->value.chptr))
+			    {
+				if (!IsMember(cptr, lp->value.chptr))
+					continue;
+				if (IsAnonymous(lp->value.chptr))
+					continue;
+				if (!IsQuiet(lp->value.chptr))
 				    {
 #ifndef DEBUGMODE
 					if (!len) /* This saves little cpu,
@@ -937,6 +882,7 @@ void	sendto_common_channels(aClient *user, char *pattern, ...)
 							   len);
 					break;
 				    }
+			    }
 		    }
 	    }
 	else
@@ -964,13 +910,15 @@ void	sendto_common_channels(aClient *user, char *pattern, ...)
 		    {
 			if (IsQuiet(channels->value.chptr))
 				continue;
-			for (lp=channels->value.chptr->members;lp;
+			if (IsAnonymous(channels->value.chptr))
+				continue;
+			for (lp=channels->value.chptr->clist;lp;
 			     lp=lp->next)
 			    {
 				cptr = lp->value.cptr;
 				if (user == cptr)
 					continue;
-				if (!MyConnect(cptr) || sentalong[cptr->fd])
+				if (!cptr->user || sentalong[cptr->fd])
 					continue;
 				sentalong[cptr->fd]++;
 #ifndef DEBUGMODE
@@ -1041,7 +989,7 @@ void	sendto_channel_butserv(aChannel *chptr, aClient *from, char *pattern, ...)
 		lfrm = &anon;
 	    }
 
-	for (lp = chptr->members; lp; lp = lp->next)
+	for (lp = chptr->clist; lp; lp = lp->next)
 		if (MyClient(acptr = lp->value.cptr) && acptr != from)
 		    {
 			if (!len)
@@ -1144,25 +1092,27 @@ void	sendto_match_servs(aChannel *chptr, aClient *from, char *format, ...)
 
 #if ! USE_STDARG
 /*VARARGS*/
-void	sendto_match_servs_v(chptr, from, ver, format, p1, p2, p3, p4, p5, p6,
-			     p7, p8, p9, p10, p11)
+int
+sendto_match_servs_v(chptr, from, ver, format, p1, p2, p3, p4, p5, p6,
+		     p7, p8, p9, p10, p11)
 aChannel *chptr;
 aClient	*from;
 char	*format, *p1, *p2, *p3, *p4, *p5, *p6, *p7, *p8, *p9, *p10, *p11;
 int	ver;
 #else
-void	sendto_match_servs_v(aChannel *chptr, aClient *from, int ver,
-			     char *format, ...)
+int
+sendto_match_servs_v(aChannel *chptr, aClient *from, int ver,
+		     char *format, ...)
 #endif
 {
-	Reg	int	i, len=0;
+	Reg	int	i, len=0, rc=0;
 	Reg	aClient	*cptr;
 	char	*mask;
 
 	if (chptr)
 	    {
 		if (*chptr->chname == '&')
-			return;
+			return 0;
 		if ((mask = (char *)rindex(chptr->chname, ':')))
 			mask++;
 	    }
@@ -1176,11 +1126,14 @@ void	sendto_match_servs_v(aChannel *chptr, aClient *from, int ver,
 			continue;
 		if (!BadPtr(mask) && match(mask, cptr->name))
 			continue;
-		if ((ver & cptr->serv->version) == 0)
-			continue;
 		if (chptr &&
 		    *chptr->chname == '!' && !(cptr->serv->version & SV_NJOIN))
 			continue;
+		if ((ver & cptr->serv->version) == 0)
+		    {
+			rc = 1;
+			continue;
+		    }
 		if (!len)
 		    {
 #if ! USE_STDARG
@@ -1195,6 +1148,68 @@ void	sendto_match_servs_v(aChannel *chptr, aClient *from, int ver,
 		    }
 		(void)send_message(cptr, sendbuf, len);
 	    }
+	return rc;
+}
+
+#if ! USE_STDARG
+/*VARARGS*/
+int
+sendto_match_servs_notv(chptr, from, ver, format, p1, p2, p3, p4, p5,
+			p6, p7, p8, p9, p10, p11)
+aChannel *chptr;
+aClient	*from;
+char	*format, *p1, *p2, *p3, *p4, *p5, *p6, *p7, *p8, *p9, *p10, *p11;
+int	ver;
+#else
+int
+sendto_match_servs_notv(aChannel *chptr, aClient *from, int ver,
+			char *format, ...)
+#endif
+{
+	Reg	int	i, len=0, rc=0;
+	Reg	aClient	*cptr;
+	char	*mask;
+
+	if (chptr)
+	    {
+		if (*chptr->chname == '&')
+			return 0;
+		if ((mask = (char *)rindex(chptr->chname, ':')))
+			mask++;
+	    }
+	else
+		mask = (char *)NULL;
+
+	for (i = fdas.highest; i >= 0; i--)
+	    {
+		if (!(cptr = local[fdas.fd[i]]) || (cptr == from) ||
+		    IsMe(cptr))
+			continue;
+		if (!BadPtr(mask) && match(mask, cptr->name))
+			continue;
+		if (chptr &&
+		    *chptr->chname == '!' && !(cptr->serv->version & SV_NJOIN))
+			continue;
+		if ((ver & cptr->serv->version) != 0)
+		    {
+			rc = 1;
+			continue;
+		    }
+		if (!len)
+		    {
+#if ! USE_STDARG
+			len = sendprep(format, p1, p2, p3, p4, p5, p6, p7,
+				       p8, p9, p10, p11);
+#else
+			va_list	va;
+			va_start(va, format);
+			len = vsendprep(format, va);
+			va_end(va);
+#endif
+		    }
+		(void)send_message(cptr, sendbuf, len);
+	    }
+	return rc;
 }
 
 /*
@@ -1215,9 +1230,9 @@ void	sendto_match_butone(aClient *one, aClient *from, char *mask, int what, char
 
 #endif
 {
-	Reg	int	i;
-	Reg	aClient *cptr, *acptr = NULL;
-	Reg	anUser	*user;
+	int	i;
+	aClient *cptr,
+		*srch;
   
 	for (i = 0; i <= highest_fd; i++)
 	    {
@@ -1227,12 +1242,24 @@ void	sendto_match_butone(aClient *one, aClient *from, char *mask, int what, char
 			continue;
 		if (IsServer(cptr))
 		    {
-			for (user = usrtop; user; user = user->nextu)
-				if (IsRegisteredUser(acptr = user->bcptr) &&
-				    acptr->from == cptr &&
-				    match_it(acptr, mask, what))
+			/*
+			** we can save some CPU here by not searching the
+			** entire list of users since it is ordered!
+			** original idea/code from pht.
+			** it could be made better by looping on the list of
+			** servers to avoid non matching blocks in the list
+			** (srch->from != cptr), but then again I never
+			** bothered to worry or optimize this routine -kalt
+			*/
+			for (srch = cptr->prev; srch; srch = srch->prev)
+			{
+				if (!IsRegisteredUser(srch))
+					continue;
+				if (srch->from == cptr &&
+				    match_it(srch, mask, what))
 					break;
-			if (!user)
+			}
+			if (srch == NULL)
 				continue;
 		    }
 		/* my client, does he match ? */
@@ -1432,101 +1459,121 @@ void	sendto_flag(u_int chan, char *pattern, ...)
 	return;
 }
 
+static int userlog;
+static int connlog;
+
+void
+logfiles_open()
+{
+#ifdef  FNAME_USERLOG
+	userlog = open(FNAME_USERLOG, O_WRONLY|O_APPEND|O_NDELAY);
+#else
+	userlog = -1;
+#endif
+#ifdef  FNAME_CONNLOG
+	connlog = open(FNAME_CONNLOG, O_WRONLY|O_APPEND|O_NDELAY);
+#else
+	connlog = -1;
+#endif
+}
+
+void
+logfiles_close()
+{
+#ifdef FNAME_USERLOG
+	if (userlog != -1)
+	{
+		(void)close(userlog);
+	}
+#endif
+#ifdef FNAME_CONNLOG
+	if (connlog != -1)
+	{
+		(void)close(connlog);
+	}
+#endif
+}
+
 /*
  * sendto_flog
  *	cptr		used for firsttime, auth, exitc, send/received M/K
- *	msg		replaces duration if duration is 0
- *	duration	only used if non 0
+ *	msg		if this contains a message, it means rejected client,
+ *			hence we will log it in connlog; otherwise it's client
+ *			quitting, so it ends up in userlog --Beeth
  *	username	can't get it from cptr
  *	hostname	i.e.
  */
-void	sendto_flog(cptr, msg, duration, username, hostname)
+void	sendto_flog(cptr, msg, username, hostname)
 aClient	*cptr;
 char	*msg, *username, *hostname;
-time_t	duration;
 {
-	char	linebuf[1024]; /* auth reply might be long.. */
+	/* 
+	** One day we will rewrite linebuf to malloc()s, but for now
+	** we are lazy. The longest linebuf I saw during last year
+	** was 216. Max auth reply can be 1024, see rfc931_work() and
+	** if iauth is disabled, read_authports() makes it max 513.
+	** And the rest... just count, I got 154 --Beeth
+	*/
+	char	linebuf[1500];
+	/*
+	** This is a potential buffer overflow.
+	** I mean, when you manage to keep ircd
+	** running for almost 12 years ;-) --B.
+	*/
+	char	buf[12];
 	int	logfile;
 
+	logfile = msg ? connlog : userlog;
+
+#if !defined(USE_SERVICES) && !( defined(USE_SYSLOG) && \
+	(defined(SYSLOG_USERS) || defined(SYSLOG_CONN)) )
+	if (logfile == -1)
+	{
+		return;
+	}
+#endif
+	if (!msg)
+	{
+		time_t	duration;
+
+		duration = timeofday - cptr->firsttime + 1;
+		(void)sprintf(buf, "%3d:%02d:%02d",
+			(int) (duration / 3600),
+			(int) ((duration % 3600) / 60),
+			(int) (duration % 60));
+	}
+
+	/*
+	** Aha, cptr->firsttime is time when user connected,
+	** so we syslog() it anyway. I'm waving big
+	** "rewrite log format" flag --Beeth.
+	*/
+	(void)sprintf(linebuf,
+		"%s (%s): %s@%s [%s] %c %lu %luKb %lu %luKb\n",
+		myctime(cptr->firsttime), msg ? msg : buf,
+		username, hostname, cptr->auth,
+		cptr->exitc, cptr->sendM, cptr->sendK,
+		cptr->receiveM, cptr->receiveK);
+
+#if defined(USE_SYSLOG) && (defined(SYSLOG_USERS) || defined(SYSLOG_CONN))
+	syslog(LOG_NOTICE, "%s", linebuf);
+#endif
+
 #ifdef	USE_SERVICES
-	if (duration)
-	    {
-		(void)sprintf(linebuf,
-	      "%s (%3d:%02d:%02d): %s@%s [%s] %c %lu %luKb %lu %luKb\n",
-			      myctime(cptr->firsttime),
-			      (int) (duration / 3600),
-			      (int) ((duration % 3600) / 60),
-			      (int) (duration % 60),
-			      username, hostname, cptr->auth,
-			      cptr->exitc, cptr->sendM, cptr->sendK,
-			      cptr->receiveM, cptr->receiveK);
+	if (!msg)
+	{
 		check_services_butone(SERVICE_WANT_USERLOG, NULL, &me,
 				      "USERLOG :%s", linebuf);
-	    }
+	}
 	else
-	    {
-		(void)sprintf(linebuf,
-			      "%s (%s): %s@%s [%s] %c %lu %luKb %lu %luKb\n",
-			      myctime(cptr->firsttime), msg, username,
-			      hostname, cptr->auth,
-			      cptr->exitc, cptr->sendM, cptr->sendK,
-			      cptr->receiveM, cptr->receiveK);
+	{
 		check_services_butone(SERVICE_WANT_CONNLOG, NULL, &me,
 				      "CONNLOG :%s", linebuf);
-	    }
+	}
 #endif
-	/*
-	 * This conditional makes the logfile active only after
-	 * it's been created, thus logging can be turned off by
-	 * removing the file.
-	 *
-	 * stop NFS hangs...most systems should be able to
-	 * file in 3 seconds. -avalon (curtesy of wumpus)
-	 */
-	(void)alarm(3);
-	if (
-#ifdef	FNAME_USERLOG
-	    (duration && 
-	     (logfile = open(FNAME_USERLOG, O_WRONLY|O_APPEND)) != -1)
-# ifdef	FNAME_CONNLOG
-	    ||
-# endif
-#endif
-#ifdef	FNAME_CONNLOG
-	    (!duration && 
-	     (logfile = open(FNAME_CONNLOG, O_WRONLY|O_APPEND)) != -1)
-#else
-# ifndef	FNAME_USERLOG
-	    0
-# endif
-#endif
-	   )
-	    {
-		(void)alarm(0);
-#ifndef	USE_SERVICES
-		if (duration)
-			(void)sprintf(linebuf,
-	      "%s (%3d:%02d:%02d): %s@%s [%s] %c %lu %luKb %lu %luKb\n",
-				      myctime(cptr->firsttime),
-				      (int) (duration / 3600),
-				      (int) ((duration % 3600) / 60),
-				      (int) (duration % 60),
-				      username, hostname, cptr->auth,
-				      cptr->exitc, cptr->sendM, cptr->sendK,
-				      cptr->receiveM, cptr->receiveK);
-		else
-			(void)sprintf(linebuf,
-			      "%s (%s): %s@%s [%s] %c %lu %luKb %lu %luKb\n",
-				      myctime(cptr->firsttime), msg, username,
-				      hostname, cptr->auth,
-                                      cptr->exitc, cptr->sendM, cptr->sendK,
-                                      cptr->receiveM, cptr->receiveK);
-#endif
-		(void)alarm(3);
+	if (logfile != -1)
+	{
 		(void)write(logfile, linebuf, strlen(linebuf));
-		(void)alarm(0);
-		(void)close(logfile);
-	    }
-	(void)alarm(0);
+	}
 }
 #endif /* CLIENT_COMPILE */
