@@ -37,16 +37,20 @@ struct Module
 struct Instance
 {
     AnInstance	*nexti;
+    u_char	in;			/* instance number */
     aModule	*mod;			/* module */
     char	*opt;			/* options read from file */
     char	*popt;			/* options to send to ircd */
     void	*data;			/* private data: stats, ... */
     aTarget	*address;
     aTarget	*hostname;
+    u_int	timeout;
 };
 
 struct Target
 {
     char	*value;
+    u_long	baseip, lmask;		/* a.b.c.d/z */
+    char	yes;
     aTarget	*nextt;
 };
