@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: iauth.c,v 1.7 1999/01/13 02:14:36 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: iauth.c,v 1.11.2.1 2001/05/16 10:42:51 chopin Exp $";
 #endif
 
 #include "os.h"
@@ -140,13 +140,19 @@ int	argc;
 char	*argv[];
 {
 	time_t	nextst = time(NULL) + 90;
+	char *xopt;
 
 	if (argc == 2 && !strcmp(argv[1], "-X"))
 		exit(0);
 
 	if (isatty(0))
 	    {
-		(void)printf("iauth %s\n", make_version());
+		(void)printf("iauth %s", make_version());
+#if defined(USE_DSM)
+			(void)printf(" (with DSM support)\n");
+#else
+			(void)printf("\n");
+#endif
 		if (argc == 3 && !strcmp(argv[1], "-c"))
 		    {
 			(void)printf("\nReading \"%s\"\n\n", argv[2]);
@@ -169,6 +175,7 @@ char	*argv[];
 
 	init_signals();
 	init_syslog();
+	xopt = conf_read(NULL);
 	init_filelogs();
 	sendto_log(ALOG_DMISC, LOG_NOTICE, "Daemon starting (%s%s).",
 		   make_version(),
@@ -179,7 +186,8 @@ char	*argv[];
 #endif
 		   );
 	init_io();
-	conf_read(NULL);
+	sendto_ircd("V %s", make_version());
+	sendto_ircd("O %s", xopt);
 	conf_ircd();
 
 #if defined(IAUTH_DEBUG)
