@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: c_msg.c,v 1.2 1997/09/03 17:45:34 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: c_msg.c,v 1.3.2.1 2001/02/27 20:00:35 q Exp $";
 #endif
  
 #include "os.h"
@@ -120,13 +120,14 @@ char *parv[];
   putline(mybuf);
 }
 
-void m_server(sptr, cptr, parc, parv)
+int m_server(sptr, cptr, parc, parv)
 aClient *cptr, *sptr;
 int parc;
 char *parv[];
 {
   sprintf(mybuf,"*** New server: %s", parv[1]);
   putline(mybuf);
+  return 0;
 }
 
 int m_topic(sptr, cptr, parc, parv)
@@ -152,6 +153,8 @@ char	*parv[];
 	sprintf(mybuf,"*** %s <%s> joined channel %s", 
 		parv[0], userhost, parv[1]);
 	putline(mybuf);
+	free(querychannel);
+	querychannel = mystrdup(parv[1]);
   return 0;
 }
 
