@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: irc.c,v 1.4 1998/04/25 00:30:11 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: irc.c,v 1.6.4.1 1999/10/30 19:28:37 q Exp $";
 #endif
  
 #include "os.h"
@@ -157,6 +157,7 @@ char	*argv[];
 	channel[0] = '\0';
 	me.user = &meUser;
 	me.from = &me;
+	me.info = (char *) malloc(REALLEN + 1);
 	setuid(getuid());
 	version = make_version();
 
@@ -631,7 +632,7 @@ char	*buf, *temp;
 
 	if (termtype == CURSES_TERM) {
 		apu = 0;
-		sprintf(header,HEADER,version, me.name, currserver);
+		sprintf(header, IRCHEADER ,version, me.name, currserver);
 		clear();
 		standout();
 		mvaddstr(LINES - 2, 0, header);
@@ -716,7 +717,7 @@ char *line;
 				refresh();
 				apu -= 4;
 				/* redraw status line */
-				sprintf(header,HEADER,version,
+				sprintf(header, IRCHEADER, version,
 					me.name, currserver);
 				standout();
 				mvaddstr(LINES - 2, 0, header);
@@ -882,7 +883,7 @@ void write_statusline()
 	char	header[HEADERLEN];
 
 	if (termtype == CURSES_TERM) {
-		sprintf(header, HEADER, version, me.name, currserver);
+		sprintf(header, IRCHEADER, version, me.name, currserver);
 		standout();
 		mvaddstr(LINES - 2, 0, header);
 		standend();
