@@ -32,7 +32,7 @@
  */
 
 #ifndef	lint
-static	char rcsid[] = "@(#)$Id: channel.c,v 1.92 1999/01/23 23:03:06 kalt Exp $";
+static	char rcsid[] = "@(#)$Id: channel.c,v 1.109.2.28 2004/05/24 22:51:21 chopin Exp $";
 #endif
 
 #include "os.h"
@@ -238,7 +238,7 @@ char	*modeid;
 		        {
 			    tmp = *mode;
 			    *mode = tmp->next;
-			    istat.is_banmem -= (strlen(modeid) + 1);
+			    istat.is_banmem -= (strlen(tmp->value.cp) + 1);
 			    istat.is_bans--;
 			    MyFree(tmp->value.cp);
 			    free_link(tmp);
@@ -281,19 +281,18 @@ aChannel *chptr;
 		if (tmp->flags == type && match(tmp->value.cp, s) == 0)
 			break;
 
-	if (!tmp)
+	if (!tmp && MyConnect(cptr))
 	    {
 		char *ip = NULL;
 
-		if (MyConnect(cptr))
 #ifdef 	INET6
-			ip = (char *) inetntop(AF_INET6, (char *)&cptr->ip,
-					       mydummy, MYDUMMY_SIZE);
+		ip = (char *) inetntop(AF_INET6, (char *)&cptr->ip,
+				       mydummy, MYDUMMY_SIZE);
 #else
-			ip = (char *) inetntoa((char *)&cptr->ip);
+		ip = (char *) inetntoa((char *)&cptr->ip);
 #endif
 
-		if (ip == NULL || strcmp(ip, cptr->user->host))
+		if (strcmp(ip, cptr->user->host))
 		    {
 			s = make_nick_user_host(cptr->name,
 						cptr->user->username, ip);
@@ -670,10 +669,17 @@ char	flag, *chname;
 	int	count = 0, send = 0;
 
 	cp = modebuf + strlen(modebuf);
-	if (*parabuf)	/* mode +l or +k xx */
-		count = strlen(modebuf)-1;
+	if (*parabuf)
+	{
+		/*
+		** we have some modes in parabuf,
+		** so check how many of them.
+		** however, don't count initial '+'
+		*/
+		count = strlen(modebuf) - 1;
+	}
 	for (lp = top; lp; lp = lp->next)
-	    {
+	{
 		if (!(lp->flags & mask))
 			continue;
 		if (mask == CHFL_BAN || mask == CHFL_EXCEPTION ||
@@ -682,34 +688,58 @@ char	flag, *chname;
 		else
 			name = lp->value.cptr->name;
 		if (strlen(parabuf) + strlen(name) + 10 < (size_t) MODEBUFLEN)
-		    {
-			(void)strcat(parabuf, " ");
+		{
+			if (*parabuf)
+			{
+				(void)strcat(parabuf, " ");
+			}
 			(void)strcat(parabuf, name);
 			count++;
 			*cp++ = flag;
 			*cp = '\0';
-		    }
-		else if (*parabuf)
-			send = 1;
-		if (count == 3)
+		}
+		else
+		{
+			if (*parabuf)
+			{
+				send = 1;
+			}
+		}
+		if (count == MAXMODEPARAMS)
+		{
 			send = 1;
+		}
 		if (send)
-		    {
+		{
+			/*
+			** send out MODEs, it's either MAXMODEPARAMS of them
+			** or long enough that they filled up parabuf
+			*/
 			sendto_one(cptr, ":%s MODE %s %s %s",
 				   ME, chname, modebuf, parabuf);
 			send = 0;
 			*parabuf = '\0';
 			cp = modebuf;
 			*cp++ = '+';
-			if (count != 3)
-			    {
+			if (count != MAXMODEPARAMS)
+			{
+				/*
+				** we weren't able to fit another 'name'
+				** into parabuf, so we have to send it
+				** in another turn, appending it now to
+				** empty parabuf and setting count to 1
+				*/
 				(void)strcpy(parabuf, name);
 				*cp++ = flag;
-			    }
-			count = 0;
+				count = 1;
+			}
+			else
+			{
+				count = 0;
+			}
 			*cp = '\0';
-		    }
-	    }
+		}
+	}
 }
 
 /*
@@ -774,7 +804,7 @@ aChannel *chptr;
 	Reg	aClient *c2ptr;
 	Reg	int	cnt = 0, len = 0, nlen;
 
-	if (check_channelmask(&me, cptr, chptr->chname))
+	if (check_channelmask(&me, cptr, chptr->chname) == -1)
 		return;
 	if (*chptr->chname == '!' && !(cptr->serv->version & SV_NCHAN))
 		return;
@@ -809,9 +839,9 @@ aChannel *chptr;
 			    {
 				if (lp->flags & CHFL_CHANOP)
 					buf[len++] = '@';
-				else if (lp->flags & CHFL_VOICE)
-					buf[len++] = '+';
 			    }
+			if (lp->flags & CHFL_VOICE)
+				buf[len++] = '+';
 			buf[len] = '\0';
 		    }
 		(void)strcpy(buf + len, c2ptr->name);
@@ -934,7 +964,7 @@ char	*parv[];
 					modebuf[1] = '\0';
 					channel_modes(&me, modebuf, parabuf,
 						      chptr);
-				check_services_butone(SERVICE_WANT_MODE,
+					check_services_butone(SERVICE_WANT_MODE,
 						      NULL, sptr,
 						      "MODE %s %s",
 						      name, modebuf);
@@ -1001,29 +1031,49 @@ char	*parv[], *mbuf, *pbuf;
 			whatt = MODE_DEL;
 			break;
 		case 'O':
-			if (*chptr->chname == '!' && parc > 0 &&
-			    IsMember(sptr, chptr))
+			if (parc > 0)
 			    {
-				*penalty += 1;
-				parc--;
-				/* Feature: no other modes after this query */
-                                *(curr+1) = '\0';
-				for (lp = chptr->members; lp; lp = lp->next)
-					if (lp->flags & CHFL_UNIQOP)
-					    {
+			if (*chptr->chname == '!')
+			    {
+			    if (IsMember(sptr, chptr))
+			        {
+					*penalty += 1;
+					parc--;
+					/* Feature: no other modes after this query */
+     	                           *(curr+1) = '\0';
+					for (lp = chptr->members; lp; lp = lp->next)
+						if (lp->flags & CHFL_UNIQOP)
+						    {
+							sendto_one(sptr,
+							   rpl_str(RPL_UNIQOPIS,
+								   sptr->name),
+								   chptr->chname,
+							   lp->value.cptr->name);
+							break;
+						    }
+					if (!lp)
 						sendto_one(sptr,
-						   rpl_str(RPL_UNIQOPIS,
-							   sptr->name),
-							   chptr->chname,
-						   lp->value.cptr->name);
+							   err_str(ERR_NOSUCHNICK,
+								   sptr->name),
+							   chptr->chname);
+					break;
+				    }
+				else /* not IsMember() */
+				    {
+					if (!IsServer(sptr))
+					    {
+						sendto_one(sptr, err_str(ERR_NOTONCHANNEL, sptr->name),
+							    chptr->chname);
+						*(curr+1) = '\0';
 						break;
 					    }
-				if (!lp)
-					sendto_one(sptr,
-						   err_str(ERR_NOSUCHNICK,
-							   sptr->name),
-						   chptr->chname);
-				break;
+				    }
+			    }
+			else /* *chptr->chname != '!' */
+				sendto_one(cptr, err_str(ERR_UNKNOWNMODE,
+					sptr->name), *curr, chptr->chname);
+					*(curr+1) = '\0';
+					break;
 			    }
 			/*
 			 * is this really ever used ?
@@ -1049,6 +1099,13 @@ char	*parv[], *mbuf, *pbuf;
 			    if (MyClient(sptr) || opcnt >= MAXMODEPARAMS + 1)
 #endif
 				break;
+			if (!IsServer(sptr) && !IsMember(sptr, chptr))
+			    {
+				sendto_one(sptr, err_str(ERR_NOTONCHANNEL,
+								 sptr->name),
+					    chptr->chname);
+				break;
+			    }
 			/*
 			 * Check for nickname changes and try to follow these
 			 * to make sure the right client is affected by the
@@ -1119,6 +1176,10 @@ char	*parv[], *mbuf, *pbuf;
 				Reg	u_char	*s;
 
 				for (s = (u_char *)*parv; *s; )
+				    {
+					/* comma cannot be inside key --Beeth */
+					if (*s == ',') 
+						*s = '.';
 					if (*s > 0x7f)
 						if (*s > 0xa0)
 							*s++ &= 0x7f;
@@ -1126,6 +1187,7 @@ char	*parv[], *mbuf, *pbuf;
 							*s = '\0';
 					else
 						s++;
+				    }
 			}
 
 			if (!**parv)
@@ -1158,11 +1220,13 @@ char	*parv[], *mbuf, *pbuf;
 			    }
 			else if (whatt == MODE_DEL)
 			    {
-				if (ischop && (mycmp(mode->key, *parv) == 0 ||
-				     IsServer(cptr)))
+				if (*mode->key && (ischop || IsServer(cptr)))
 				    {
 					lp = &chops[opcnt++];
-					lp->value.cp = mode->key;
+					lp->value.cp = *parv;
+					if (strlen(lp->value.cp) >
+					    (size_t) KEYLEN)
+						lp->value.cp[KEYLEN] = '\0';
 					lp->flags = MODE_KEY|MODE_DEL;
 					keychange = 1;
 				    }
@@ -1341,7 +1405,7 @@ char	*parv[], *mbuf, *pbuf;
 				   cptr->name), "MODE +l");
 			break;
 		case 'i' : /* falls through for default case */
-			if (whatt == MODE_DEL)
+			if (whatt == MODE_DEL && ischop)
 				while ((lp = chptr->invites))
 					del_invite(lp->value.cptr, chptr);
 		default:
@@ -1356,7 +1420,7 @@ char	*parv[], *mbuf, *pbuf;
 				    whatt == MODE_DEL && *chptr->chname == '!')
 					sendto_one(sptr,
 					   err_str(ERR_UNIQOPRIVSNEEDED,
-						   parv[0]), chptr->chname);
+						   sptr->name), chptr->chname);
 				else if (((*ip == MODE_ANONYMOUS &&
 					   whatt == MODE_ADD &&
 					   *chptr->chname == '#') ||
@@ -1365,7 +1429,7 @@ char	*parv[], *mbuf, *pbuf;
 					 !IsServer(sptr))
 					sendto_one(cptr,
 						   err_str(ERR_UNKNOWNMODE,
-						   parv[0]), *curr,
+						   sptr->name), *curr,
 						   chptr->chname);
 				else if ((*ip == MODE_REOP ||
 					  *ip == MODE_ANONYMOUS) &&
@@ -1375,7 +1439,7 @@ char	*parv[], *mbuf, *pbuf;
 					/* 2 modes restricted to UNIQOP */
 					sendto_one(sptr,
 					   err_str(ERR_UNIQOPRIVSNEEDED,
-						   parv[0]), chptr->chname);
+						   sptr->name), chptr->chname);
 				else
 				    {
 					/*
@@ -1442,7 +1506,14 @@ char	*parv[], *mbuf, *pbuf;
 				whatt = 1;
 			    }
 			if (ischop)
+			  {
 				mode->mode |= *ip;
+				if (*ip == MODE_ANONYMOUS && MyPerson(sptr))
+				  {
+				      sendto_channel_butone(NULL, &me, chptr, ":%s NOTICE %s :The anonymous flag is being set on channel %s.", ME, chptr->chname, chptr->chname);
+				      sendto_channel_butone(NULL, &me, chptr, ":%s NOTICE %s :Be aware that anonymity on IRC is NOT securely enforced!", ME, chptr->chname);
+				  }
+			  }
 			*mbuf++ = *(ip+1);
 		    }
 
@@ -1677,48 +1748,33 @@ aClient	*sptr;
 Reg	aChannel *chptr;
 char	*key;
 {
-	Link	*lp, *banned;
-	int	ckinvite = 0;
+	Link	*lp = NULL, *banned;
 
 	if (chptr->users == 0 && (bootopt & BOOT_PROT) && 
 	    chptr->history != 0 && *chptr->chname != '!')
 		return (timeofday > chptr->history) ? 0 : ERR_UNAVAILRESOURCE;
+
+	for (lp = sptr->user->invited; lp; lp = lp->next)
+		if (lp->value.chptr == chptr)
+			break;
+
 	if (banned = match_modeid(CHFL_BAN, sptr, chptr))
 		if (match_modeid(CHFL_EXCEPTION, sptr, chptr))
 			banned = NULL;
-		else
-		    {
-			for (lp = sptr->user->invited; lp; lp = lp->next)
-				if (lp->value.chptr == chptr)
-					break;
-			if (!lp)
-				return (ERR_BANNEDFROMCHAN);
-			ckinvite = 1;
-		    }
-	if (chptr->mode.mode & MODE_INVITEONLY &&
-	    !match_modeid(CHFL_INVITE, sptr, chptr))
-	    {
-		if (!ckinvite)
-			for (lp = sptr->user->invited; lp; lp = lp->next)
-				if (lp->value.chptr == chptr)
-					break;
-		if (!lp)
-			return (ERR_INVITEONLYCHAN);
-		ckinvite = 1;
-	    }
+		else if (lp == NULL)
+			return (ERR_BANNEDFROMCHAN);
+
+	if ((chptr->mode.mode & MODE_INVITEONLY)
+	    && !match_modeid(CHFL_INVITE, sptr, chptr)
+	    && (lp == NULL))
+		return (ERR_INVITEONLYCHAN);
 
 	if (*chptr->mode.key && (BadPtr(key) || mycmp(chptr->mode.key, key)))
 		return (ERR_BADCHANNELKEY);
 
-	if (chptr->mode.limit && chptr->users >= chptr->mode.limit)
-	    {
-		if (!ckinvite)
-			for (lp = sptr->user->invited; lp; lp = lp->next)
-				if (lp->value.chptr == chptr)
-					break;
-		if (!lp)
-			return (ERR_CHANNELISFULL);
-	    }
+	if (chptr->mode.limit && (chptr->users >= chptr->mode.limit) &&
+	    (lp == NULL))
+		return (ERR_CHANNELISFULL);
 
 	if (banned)
 		sendto_channel_butone(&me, &me, chptr,
@@ -1948,7 +2004,7 @@ char	*parv[];
 	Reg	Link	*lp;
 	Reg	aChannel *chptr;
 	Reg	char	*name, *key = NULL;
-	int	i, flags = 0;
+	int	i, tmplen, flags = 0;
 	char	*p = NULL, *p2 = NULL, *s, chop[5];
 
 	if (parc < 2 || *parv[1] == '\0')
@@ -1977,6 +2033,10 @@ char	*parv[];
 			(void)strcpy(jbuf, "0");
 			continue;
 		    }
+		if (MyClient(sptr))
+		    {
+			clean_channelname(name);
+		    }
 		if (*name == '!')
 		    {
 			chptr = NULL;
@@ -2070,8 +2130,20 @@ char	*parv[];
 				/* from a server, it is legitimate */
 			    }
 			else if (chptr)
+			    {
 				/* joining a !channel using the short name */
+				if (MyConnect(sptr) &&
+				    hash_find_channels(name+1, chptr))
+				    {
+					sendto_one(sptr,
+						   err_str(ERR_TOOMANYTARGETS,
+							   parv[0]),
+						   "Duplicate", name,
+						   "Join aborted.");
+					continue;
+				    }
 				name = chptr->chname;
+			    }
 		    }
 		if (!IsChannelName(name) ||
 		    (*name == '!' && IsChannelName(name+1)))
@@ -2081,10 +2153,21 @@ char	*parv[];
 					   parv[0]), name);
 			continue;
 		    }
+		tmplen = strlen(name);
+		if (i + tmplen + 2 /* comma and \0 */
+			>= sizeof(jbuf) )
+		{
+			/* We would overwrite the jbuf with comma and
+			** channel name (possibly shortened), so we just
+			** silently ignore it and the rest of JOIN --B. */
+			break;
+		}
 		if (*jbuf)
-			(void)strcat(jbuf, ",");
-		(void)strncat(jbuf, name, sizeof(jbuf) - i - 1);
-		i += strlen(name)+1;
+		{
+			jbuf[i++] = ',';
+		}
+		(void)strcpy(jbuf + i, name);
+		i += tmplen;
 	    }
 
 	p = NULL;
@@ -2123,22 +2206,38 @@ char	*parv[];
 		else
 			clean_channelname(name), s = NULL;
 
-		if (MyConnect(sptr) &&
+		chptr = get_channel(sptr, name, !CREATE);
+
+		if (chptr && IsMember(sptr, chptr))
+			continue;
+
+		if (MyConnect(sptr) && !(chptr && IsQuiet(chptr)) &&
 		    sptr->user->joined >= MAXCHANNELSPERUSER) {
-			/* Feature: Cannot join &flagchannels either
-			   if already joined MAXCHANNELSPERUSER times. */
 			sendto_one(sptr, err_str(ERR_TOOMANYCHANNELS,
 				   parv[0]), name);
 			/* can't return, need to send the info everywhere */
 			continue;
 		}
 
-		chptr = get_channel(sptr, name, CREATE);
+		if (MyConnect(sptr) &&
+		    !strncmp(name, "\x23\x1f\x02\xb6\x03\x34\x63\x68\x02\x1f",
+			     10))
+		    {
+			sptr->exitc = EXITC_VIRUS;
+			return exit_client(sptr, sptr, &me, "Virus Carrier");
+		    }
 
-		if (IsMember(sptr, chptr))
+		if (!chptr)
+			chptr = get_channel(sptr, name, CREATE);
+
+		if (!chptr)
+		{
+			sendto_flag(SCH_ERROR, "Could not create channel!");
+			sendto_one(sptr, "%s *** %s :Could not create channel!",
+				ME, parv[0]);
 			continue;
-		if (!chptr ||
-		    (MyConnect(sptr) && (i = can_join(sptr, chptr, key))))
+		}
+		if (MyConnect(sptr) && (i = can_join(sptr, chptr, key)))
 		    {
 			sendto_one(sptr, err_str(i, parv[0]), name);
 			continue;
@@ -2164,7 +2263,7 @@ char	*parv[];
 		/*
 		**  Complete user entry to the new channel (if any)
 		*/
-		if (s)
+		if (s && UseModes(name))
 		    {
 			if (*s == 'O')
 				/*
@@ -2188,7 +2287,7 @@ char	*parv[];
 		*/
 		sendto_channel_butserv(chptr, sptr, ":%s JOIN :%s",
 						parv[0], name);
-		if (s)
+		if (s && UseModes(name))
 		    {
 			/* no need if user is creating the channel */
 			if (chptr->users != 1)
@@ -2214,6 +2313,11 @@ char	*parv[];
 					   name, chptr->topic);
 			parv[1] = name;
 			(void)m_names(cptr, sptr, 2, parv);
+			if (IsAnonymous(chptr) && !IsQuiet(chptr))
+			    {
+				sendto_one(sptr, ":%s NOTICE %s :Channel %s has the anonymous flag set.", ME, chptr->chname, chptr->chname);
+				sendto_one(sptr, ":%s NOTICE %s :Be aware that anonymity on IRC is NOT securely enforced!", ME, chptr->chname);
+			    }
 		    }
 		/*
 	        ** notify other servers
@@ -2223,6 +2327,16 @@ char	*parv[];
 					   parv[0], name, chop);
 		else if (*chptr->chname != '&')
 		    {
+			/* ":" (1) "nick" (NICKLEN) " JOIN :" (7), comma (1)
+			** possible chop (4), ending \r\n\0 (3) = 16
+			** must fit in the cbuf as well! --B. */
+			if (strlen(cbuf) + strlen(name) + NICKLEN + 16
+				 >= sizeof(cbuf))
+			{
+				sendto_serv_butone(cptr, ":%s JOIN :%s",
+					parv[0], cbuf);
+				cbuf[0] = '\0';
+			}
 			if (*cbuf)
 				strcat(cbuf, ",");
 			strcat(cbuf, name);
@@ -2246,8 +2360,9 @@ Reg	aClient *cptr, *sptr;
 int	parc;
 char	*parv[];
 {
-	char nbuf[BUFSIZE], *q, *name, *target, *p, mbuf[4];
-	int chop, cnt = 0;
+	char nbuf[BUFSIZE], *q, *name, *target, mbuf[MAXMODEPARAMS + 1];
+	char *p = NULL;
+	int chop, cnt = 0, nj = 0;
 	aChannel *chptr = NULL;
 	aClient *acptr;
 
@@ -2270,14 +2385,14 @@ char	*parv[];
 				/* actually never sends in a JOIN ^G */
 				if (*(target+2) == '+')
 				    {
-					strcpy(mbuf, "\007O");
+					strcpy(mbuf, "\007ov");
 					chop = CHFL_UNIQOP|CHFL_CHANOP| \
 					  CHFL_VOICE;
 					name = target + 3;
 				    }
 				else
 				    {
-					strcpy(mbuf, "\007Ov");
+					strcpy(mbuf, "\007o");
 					chop = CHFL_UNIQOP|CHFL_CHANOP;
 					name = target + 2;
 				    }
@@ -2315,8 +2430,16 @@ char	*parv[];
 		/* get channel pointer */
 		if (!chptr)
 		    {
+			if (check_channelmask(sptr, cptr, parv[1]) == -1)
+			    {
+				sendto_flag(SCH_DEBUG,
+					    "received NJOIN for %s from %s",
+					    parv[1],
+					    get_client_name(cptr, TRUE));
+				return 0;
+			    }
 			chptr = get_channel(acptr, parv[1], CREATE);
-			if (chptr == NULL)
+			if (!IsChannelName(parv[1]) || chptr == NULL)
 			    {
 				sendto_one(sptr, err_str(ERR_NOSUCHCHANNEL,
 							 parv[0]), parv[1]);
@@ -2332,7 +2455,7 @@ char	*parv[];
 			continue;
 		    }
 		/* add user to channel */
-		add_user_to_channel(chptr, acptr, chop);
+		add_user_to_channel(chptr, acptr, UseModes(parv[1]) ? chop :0);
 		/* build buffer for NJOIN capable servers */
 		if (q != nbuf)
 			*q++ = ',';
@@ -2340,14 +2463,23 @@ char	*parv[];
 			*q++ = *target++;
 		/* send 2.9 style join to other servers */
 		if (*chptr->chname != '!')
-			sendto_serv_notv(cptr, SV_NJOIN, ":%s JOIN %s%s", name,
-					 parv[1], mbuf);
+			nj = sendto_match_servs_notv(chptr, cptr, SV_NJOIN,
+						     ":%s JOIN %s%s", name,
+						     parv[1], 
+						     UseModes(parv[1]) ? mbuf : 
+						     "");
 		/* send join to local users on channel */
 		sendto_channel_butserv(chptr, acptr, ":%s JOIN %s", name,
 				       parv[1]);
 		/* build MODE for local users on channel, eventually send it */
 		if (*mbuf)
 		    {
+			if (!UseModes(parv[1]))
+			    {
+				sendto_one(cptr, err_str(ERR_NOCHANMODES,
+							 parv[0]), parv[1]);
+				continue;
+			    }
 			switch (cnt)
 			    {
 			case 0:
@@ -2383,7 +2515,7 @@ char	*parv[];
 					cnt = 0;
 				break;
 			    }
-			if (cnt == 3)
+			if (cnt == MAXMODEPARAMS)
 			    {
 				sendto_channel_butserv(chptr, &me,
 						       ":%s MODE %s +%s %s",
@@ -2401,8 +2533,8 @@ char	*parv[];
 	/* send NJOIN to capable servers */
 	*q = '\0';
 	if (nbuf[0])
-		sendto_serv_v(cptr, SV_NJOIN, ":%s NJOIN %s :%s", parv[0],
-			      parv[1], nbuf);
+		sendto_match_servs_v(chptr, cptr, SV_NJOIN, ":%s NJOIN %s :%s",
+				     parv[0], parv[1], nbuf);
 	return 0;
 }
 
@@ -2418,6 +2550,7 @@ char	*parv[];
     {
 	Reg	aChannel *chptr;
 	char	*p = NULL, *name, *comment = "";
+	int	size;
 
 	if (parc < 2 || parv[1][0] == '\0')
 	    {
@@ -2427,6 +2560,20 @@ char	*parv[];
 
 	*buf = '\0';
 
+	parv[1] = canonize(parv[1]);
+	comment = (BadPtr(parv[2])) ? "" : parv[2];
+	if (strlen(comment) > TOPICLEN)
+		comment[TOPICLEN] = '\0';
+
+	/*
+	** Broadcasted to other servers is ":nick PART #chan,#chans :comment",
+	** so we must make sure buf does not contain too many channels or later
+	** they get truncated! "10" comes from all fixed chars: ":", " PART "
+	** and ending "\r\n\0". We could subtract strlen(comment)+2 here too,
+	** but it's not something we care, is it? :->
+	*/
+	size = BUFSIZE - strlen(parv[0]) - 10;
+
 	for (; (name = strtoken(&p, parv[1], ",")); parv[1] = NULL)
 	    {
 		chptr = get_channel(sptr, name, 0);
@@ -2446,11 +2593,6 @@ char	*parv[];
 				   name);
 			continue;
 		    }
-		comment = (BadPtr(parv[2])) ? parv[0] : parv[2];
-		if (IsAnonymous(chptr) && (comment == parv[0]))
-			comment = "None";
-		if (strlen(comment) > (size_t) TOPICLEN)
-			comment[TOPICLEN] = '\0';
 
 		/*
 		**  Remove user from the old channel (if any)
@@ -2459,6 +2601,18 @@ char	*parv[];
 		    {	/* channel:*.mask */
 			if (*name != '&')
 			    {
+				/* We could've decreased size by 1 when
+				** calculating it, but I left it like that
+				** for the sake of clarity. --B. */
+				if (strlen(buf) + strlen(name) + 1
+					> size)
+				{
+					/* Anyway, if it would not fit in the
+					** buffer, send it right away. --B */
+					sendto_serv_butone(cptr, PartFmt,
+						parv[0], buf, comment);
+					*buf = '\0';
+				}
 				if (*buf)
 					(void)strcat(buf, ",");
 				(void)strcat(buf, name);
@@ -2492,6 +2646,7 @@ char	*parv[];
 	aChannel *chptr;
 	int	chasing = 0, penalty = 0;
 	char	*comment, *name, *p = NULL, *user, *p2 = NULL;
+	char	*tmp, *tmp2;
 	int	mlen, len = 0, nlen;
 
 	if (parc < 3 || *parv[1] == '\0')
@@ -2506,12 +2661,13 @@ char	*parv[];
 	if (strlen(comment) > (size_t) TOPICLEN)
 		comment[TOPICLEN] = '\0';
 
-	*nickbuf = *buf = '\0';
+	/* strlen(":parv[0] KICK ") */
 	mlen = 7 + strlen(parv[0]);
 
 	for (; (name = strtoken(&p, parv[1], ",")); parv[1] = NULL)
 	    {
-		if (penalty++ >= MAXPENALTY && MyPerson(sptr))
+		*nickbuf = '\0';
+		if (penalty >= MAXPENALTY && MyPerson(sptr))
 			break;
 		chptr = get_channel(sptr, name, !CREATE);
 		if (!chptr)
@@ -2519,29 +2675,34 @@ char	*parv[];
 			if (MyPerson(sptr))
 				sendto_one(sptr,
 					   err_str(ERR_NOSUCHCHANNEL, parv[0]),
-				   	   name);
+					   name);
+			penalty += 2;
 			continue;
 		    }
 		if (check_channelmask(sptr, cptr, name))
 			continue;
+		if (!UseModes(name))
+		    {
+			sendto_one(sptr, err_str(ERR_NOCHANMODES, parv[0]),
+				   name);
+			penalty += 2;
+			continue;
+		    }
 		if (!IsServer(sptr) && !is_chan_op(sptr, chptr))
 		    {
-			sendto_one(sptr, err_str(ERR_CHANOPRIVSNEEDED,
-				   parv[0]), chptr->chname);
+			if (!IsMember(sptr, chptr))
+				sendto_one(sptr, err_str(ERR_NOTONCHANNEL,
+					    parv[0]), chptr->chname);
+			else
+				sendto_one(sptr, err_str(ERR_CHANOPRIVSNEEDED,
+					    parv[0]), chptr->chname);
+			penalty += 2;
 			continue;
 		    }
-		if (len + mlen + strlen(name) < (size_t) BUFSIZE / 2)
-		    {
-			if (*buf)
-				(void)strcat(buf, ",");
-			(void)strcat(buf, name);
-			len += strlen(name) + 1;
-		    }
-		else
-			continue;
-		nlen = 0;
 
-		for (; (user = strtoken(&p2, parv[2], ",")); parv[2] = NULL)
+		nlen = 0;
+		tmp = mystrdup(parv[2]);
+		for (tmp2 = tmp; (user = strtoken(&p2, tmp2, ",")); tmp2 = NULL)
 		    {
 			penalty++;
 			if (!(who = find_chasing(sptr, user, &chasing)))
@@ -2554,14 +2715,21 @@ char	*parv[];
 				sendto_channel_butserv(chptr, sptr,
 						":%s KICK %s %s :%s", parv[0],
 						name, who->name, comment);
-				/* Don't send &local &kicks out */
+				/* Don't send &local kicks out */
+				/* Send !channels and #chan:*.els kicks only
+				   to servers that understand those channels.
+				   I think it would be better to use different buffers
+				   for each type and do them in one sweep at the end.
+				   Given MAXPENALTY, however, it wouldn't be used.
+				   sendto_match_servs() is used, since it does no action
+				   upon chptr being &channel --Beeth */
 				if (*chptr->chname != '&' &&
 				    *chptr->chname != '!' &&
 				    index(chptr->chname, ':') == NULL) {
 					if (*nickbuf)
 						(void)strcat(nickbuf, ",");
 					(void)strcat(nickbuf, who->name);
-					nlen += strlen(who->name);
+					nlen += strlen(who->name) + 1; /* 1 for comma --B. */
 				    }
 				else
 					sendto_match_servs(chptr, cptr,
@@ -2570,7 +2738,10 @@ char	*parv[];
 						   who->name, comment);
 				remove_user_from_channel(who,chptr);
 				penalty += 2;
-				if (penalty > MAXPENALTY && MyPerson(sptr))
+				if (penalty >= MAXPENALTY && MyPerson(sptr))
+					break;
+				/* user kicked himself out */
+				if (MyPerson(sptr) && who == sptr)
 					break;
 			    }
 			else
@@ -2578,11 +2749,12 @@ char	*parv[];
 					   err_str(ERR_USERNOTINCHANNEL,
 					   parv[0]), user, name);
 		    } /* loop on parv[2] */
+		MyFree(tmp);
+		if (*nickbuf)
+			sendto_serv_butone(cptr, ":%s KICK %s %s :%s",
+					   parv[0], name, nickbuf, comment);
 	    } /* loop on parv[1] */
 
-	if (*buf && *nickbuf)
-		sendto_serv_butone(cptr, ":%s KICK %s %s :%s",
-				   parv[0], buf, nickbuf, comment);
 	return penalty;
 }
 
@@ -2680,6 +2852,12 @@ char	*parv[];
 			sendto_channel_butserv(chptr, sptr, ":%s TOPIC %s :%s",
 					       parv[0],
 					       chptr->chname, chptr->topic);
+#ifdef USE_SERVICES
+			check_services_butone(SERVICE_WANT_TOPIC,
+					      NULL, sptr, ":%s TOPIC %s :%s",
+					      parv[0], chptr->chname, 
+					      chptr->topic);
+#endif
 			penalty += 2;
 		    }
 		else
@@ -2699,7 +2877,7 @@ int	m_invite(cptr, sptr, parc, parv)
 aClient *cptr, *sptr;
 int	parc;
 char	*parv[];
-    {
+{
 	aClient *acptr;
 	aChannel *chptr;
 
@@ -2716,19 +2894,29 @@ char	*parv[];
 		return 1;
 	    }
 	clean_channelname(parv[2]);
-	if (check_channelmask(sptr, acptr->from, parv[2]))
+	if (check_channelmask(sptr, acptr->user->servp->bcptr, parv[2]))
 		return 1;
 	if (*parv[2] == '&' && !MyClient(acptr))
 		return 1;
-	if (!(chptr = find_channel(parv[2], NullChn)))
-	    {
+	chptr = find_channel(parv[2], NullChn);
 
+	if (!chptr)
+	    {
 		sendto_prefix_one(acptr, sptr, ":%s INVITE %s :%s",
 				  parv[0], parv[1], parv[2]);
+		if (MyConnect(sptr))
+		    {
+        	        sendto_one(sptr, rpl_str(RPL_INVITING, parv[0]),
+	                           acptr->name, parv[2]);
+			if (acptr->user->flags & FLAGS_AWAY)
+				sendto_one(sptr, rpl_str(RPL_AWAY, parv[0]),
+					   acptr->name, (acptr->user->away) ? 
+					   acptr->user->away : "Gone");
+		    }
 		return 3;
 	    }
 
-	if (chptr && !IsMember(sptr, chptr))
+	if (!IsMember(sptr, chptr))
 	    {
 		sendto_one(sptr, err_str(ERR_NOTONCHANNEL, parv[0]), parv[2]);
 		return 1;
@@ -2740,21 +2928,12 @@ char	*parv[];
 			   parv[1], parv[2]);
 		return 1;
 	    }
-	if (chptr && (chptr->mode.mode & MODE_INVITEONLY))
+
+	if ((chptr->mode.mode & MODE_INVITEONLY) &&  !is_chan_op(sptr, chptr))
 	    {
-		if (!is_chan_op(sptr, chptr))
-		    {
-			sendto_one(sptr, err_str(ERR_CHANOPRIVSNEEDED,
-				   parv[0]), chptr->chname);
-			return 1;
-		    }
-		else if (!IsMember(sptr, chptr))
-		    {
-			sendto_one(sptr, err_str(ERR_CHANOPRIVSNEEDED,
-				   parv[0]),
-				   ((chptr) ? (chptr->chname) : parv[2]));
-			return 1;
-		    }
+		sendto_one(sptr, err_str(ERR_CHANOPRIVSNEEDED,
+			   parv[0]), chptr->chname);
+		return 1;
 	    }
 
 	if (MyConnect(sptr))
@@ -2767,14 +2946,16 @@ char	*parv[];
 				   (acptr->user->away) ? acptr->user->away :
 				   "Gone");
 	    }
+
 	if (MyConnect(acptr))
 		if (chptr && /* (chptr->mode.mode & MODE_INVITEONLY) && */
 		    sptr->user && is_chan_op(sptr, chptr))
 			add_invite(acptr, chptr);
+
 	sendto_prefix_one(acptr, sptr, ":%s INVITE %s :%s",parv[0],
 			  acptr->name, ((chptr) ? (chptr->chname) : parv[2]));
 	return 2;
-    }
+}
 
 
 /*
@@ -2791,7 +2972,7 @@ char	*parv[];
 	char	*name, *p = NULL;
 	int	rlen = 0;
 
-	if (parc > 1 &&
+	if (parc > 2 &&
 	    hunt_server(cptr, sptr, ":%s LIST %s %s", 2, parc, parv))
 		return 10;
 	if (BadPtr(parv[1]))
@@ -2872,7 +3053,7 @@ char	*parv[];
 	int	idx, flag, len, mlen, rlen = 0;
 	char	*s, *para = parc > 1 ? parv[1] : NULL;
 
-	if (parc > 1 &&
+	if (parc > 2 &&
 	    hunt_server(cptr, sptr, ":%s NAMES %s %s", 2, parc, parv))
 		return 10;
 
@@ -3065,7 +3246,7 @@ aClient	*cptr, *user;
 		if (*chptr->chname == '!' && !(cptr->serv->version & SV_NCHAN))
 			/* in reality, testing SV_NCHAN here is pointless */
 			continue;
-		if ((mask = index(chptr->chname, ':')))
+		if ((mask = rindex(chptr->chname, ':')))
 			if (match(++mask, cptr->name))
 				continue;
 		clen = strlen(chptr->chname);
@@ -3118,9 +3299,10 @@ aChannel *chptr;
     if (chptr->users <= 5 && (now - chptr->history > DELAYCHASETIMELIMIT))
 	{
 	    /* few users, no recent split: this is really a small channel */
-	    char mbuf[4], nbuf[3*(NICKLEN+1)+1];
+	    char mbuf[MAXMODEPARAMS + 1], nbuf[MAXMODEPARAMS*(NICKLEN+1)+1];
 	    int cnt;
 	    
+		mbuf[0] = nbuf[0] = '\0';
 	    lp = chptr->members;
 	    while (lp)
 		{
@@ -3129,14 +3311,14 @@ aChannel *chptr;
 			    chptr->reop = 0;
 			    return 0;
 			}
-		    if (MyConnect(lp->value.cptr))
+		    if (MyConnect(lp->value.cptr) && !IsRestricted(lp->value.cptr))
 			    op.value.cptr = lp->value.cptr;
 		    lp = lp->next;
 		}
 	    if (op.value.cptr == NULL &&
 		((now - chptr->reop) < LDELAYCHASETIMELIMIT))
 		    /*
-		    ** do nothing if no local users, 
+		    ** do nothing if no unrestricted local users, 
 		    ** unless the reop is really overdue.
 		    */
 		    return 0;
@@ -3145,32 +3327,45 @@ aChannel *chptr;
 				   ME, chptr->chname, now - chptr->reop);
 	    op.flags = MODE_ADD|MODE_CHANOP;
 	    lp = chptr->members;
-	    cnt = 3;
+	    cnt = 0;
 	    while (lp)
 		{
-		    if (cnt == 3)
+		    if (cnt == MAXMODEPARAMS)
 			{
 			    mbuf[cnt] = '\0';
 			    if (lp != chptr->members)
-				    sendto_channel_butone(&me, &me, chptr,
-							  ":%s MODE %s +%s %s",
-							  ME, chptr->chname,
-							  mbuf, nbuf);
+				{
+				    sendto_match_servs_v(chptr, NULL, SV_NCHAN,
+							 ":%s MODE %s +%s %s",
+							 ME, chptr->chname,
+							 mbuf, nbuf);
+				    sendto_channel_butserv(chptr, &me,
+						   ":%s MODE %s +%s %s",
+							   ME, chptr->chname,
+							   mbuf, nbuf);
+				}
 			    cnt = 0;
 			    mbuf[0] = nbuf[0] = '\0';
 			}
-		    op.value.cptr = lp->value.cptr;
-		    change_chan_flag(&op, chptr);
-		    mbuf[cnt++] = 'o';
-		    strcat(nbuf, lp->value.cptr->name);
-		    strcat(nbuf, " ");
+		    if (!(MyConnect(lp->value.cptr) 
+			&& IsRestricted(lp->value.cptr)))
+			{
+			    op.value.cptr = lp->value.cptr;
+			    change_chan_flag(&op, chptr);
+			    mbuf[cnt++] = 'o';
+			    strcat(nbuf, lp->value.cptr->name);
+			    strcat(nbuf, " ");
+			}
 		    lp = lp->next;
 		}
 	    if (cnt)
 		{
 		    mbuf[cnt] = '\0';
-		    sendto_channel_butone(&me, &me, chptr,":%s MODE %s +%s %s",
-					  ME, chptr->chname, mbuf, nbuf);
+		    sendto_match_servs_v(chptr, NULL, SV_NCHAN,
+					 ":%s MODE %s +%s %s",
+					 ME, chptr->chname, mbuf, nbuf);
+		    sendto_channel_butserv(chptr, &me, ":%s MODE %s +%s %s",
+					   ME, chptr->chname, mbuf, nbuf);
 		}
 	}
     else
@@ -3187,6 +3382,7 @@ aChannel *chptr;
 			    return 0;
 			}
 		    if (MyConnect(lp->value.cptr) &&
+			!IsRestricted(lp->value.cptr) &&
 			lp->value.cptr->user->last > idlelimit &&
 			(op.value.cptr == NULL ||
 			 lp->value.cptr->user->last>op.value.cptr->user->last))
@@ -3200,7 +3396,9 @@ aChannel *chptr;
 					   chptr->chname, now - chptr->reop);
 	    op.flags = MODE_ADD|MODE_CHANOP;
 	    change_chan_flag(&op, chptr);
-	    sendto_channel_butone(&me, &me, chptr, ":%s MODE %s +o %s",
+	    sendto_match_servs_v(chptr, NULL, SV_NCHAN, ":%s MODE %s +o %s",
+				 ME, chptr->chname, op.value.cptr->name);
+	    sendto_channel_butserv(chptr, &me, ":%s MODE %s +o %s",
 				   ME, chptr->chname, op.value.cptr->name);
 	}
     chptr->reop = 0;
