@@ -18,26 +18,8 @@
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* -- Jto -- 20 Jun 1990
- * extern void free() fixed as suggested by
- * gruner@informatik.tu-muenchen.de
- */
-
-/* -- Jto -- 03 Jun 1990
- * Added chname initialization...
- */
-
-/* -- Jto -- 24 May 1990
- * Moved is_full() to channel.c
- */
-
-/* -- Jto -- 10 May 1990
- * Added #include <sys.h>
- * Changed memset(xx,0,yy) into bzero(xx,yy)
- */
-
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: list.c,v 1.7 1998/12/28 15:44:57 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: list.c,v 1.9.2.3 2001/05/06 21:36:55 chopin Exp $";
 #endif
 
 #include "os.h"
@@ -55,7 +37,6 @@ static	struct	liststats {
 
 #endif
 
-anUser	*usrtop = NULL;
 aServer	*svrtop = NULL;
 
 int	numclients = 0;
@@ -90,11 +71,6 @@ void	checklists()
 		if (sp->bcptr->serv != sp)
 			Debug((DEBUG_ERROR, "svrtop: %#x->%#x->%#x != %#x",
 				sp, sp->bcptr, sp->bcptr->serv, sp));
-	
-	for (up = usrtop; up; up = up->nextu)
-		if (up->bcptr->user != up)
-			Debug((DEBUG_ERROR, "usrtop: %#x->%#x->%#x != %#x",
-				up, up->bcptr, up->bcptr->user, up));
 }
 #endif
 	
@@ -164,6 +140,13 @@ aClient	*cptr;
 {
 	if (cptr->info != DefInfo)
 		MyFree(cptr->info);
+	if (MyConnect(cptr) && cptr->auth != cptr->username)
+	{
+	    sendto_flag(SCH_ERROR, "Please report to ircd-bug@irc.org about cptr->auth allocated but not free()d!");
+		istat.is_authmem -= strlen(cptr->auth) + 1;
+		istat.is_auth -= 1;
+		MyFree(cptr->auth);
+	}
 	MyFree((char *)cptr);
 }
 
@@ -191,8 +174,6 @@ aClient *cptr;
 		user->invited = NULL;
 		user->uwas = NULL;
 		cptr->user = user;
-		user->nextu = NULL;
-		user->prevu = NULL;
 		user->servp = NULL;
 		user->bcptr = cptr;
 		if (cptr->next)	/* the only cptr->next == NULL is me */
@@ -213,7 +194,6 @@ aClient	*cptr;
 		servs.inuse++;
 #endif
 		serv->user = NULL;
-		serv->userlist = NULL;
 		serv->snum = -1;
 		*serv->by = '\0';
 		*serv->tok = '\0';
@@ -279,14 +259,14 @@ aClient	*cptr;
 		 */
 		if (user->joined || user->refcnt < 0 ||
 		    user->invited || user->channel || user->uwas ||
-		    user->bcptr || user->prevu || user->nextu)
+		    user->bcptr)
 		    {
 			char buf[512];
 			/*too many arguments for dumpcore() and sendto_flag()*/
-			SPRINTF(buf, "%#x %#x %#x %#x %d %d %#x %#x %#x (%s)",
+			SPRINTF(buf, "%#x %#x %#x %#x %d %d %#x (%s)",
 				user, user->invited, user->channel, user->uwas,
 				user->joined, user->refcnt,
-				user->prevu, user->nextu, user->bcptr,
+				user->bcptr,
 				(user->bcptr) ? user->bcptr->name :"none");
 #ifdef DEBUGMODE
 			dumpcore("%#x user (%s!%s@%s) %s",
@@ -313,12 +293,12 @@ aClient	*cptr;
 	if (--serv->refcnt <= 0)
 	    {
 		if (serv->refcnt < 0 ||	serv->prevs || serv->nexts ||
-		    serv->bcptr || serv->userlist || serv->user)
+		    serv->bcptr || serv->user)
 		    {
 			char buf[512];
-			SPRINTF(buf, "%d %#x %#x %#x %#x %#x (%s)",
+			SPRINTF(buf, "%d %#x %#x %#x %#x (%s)",
 				serv->refcnt, serv->prevs, serv->nexts,
-				serv->userlist, serv->user, serv->bcptr,
+				serv->user, serv->bcptr,
 				(serv->bcptr) ? serv->bcptr->name : "none");
 #ifdef DEBUGMODE
 			dumpcore("%#x server %s %s",
@@ -360,29 +340,6 @@ Reg	aClient	*cptr;
 	if (cptr->user)
 	    {
 		istat.is_users--;
-		/*
-		** has to be removed from the list of aUser structures,
-		** be careful of user->servp->userlist
-		*/
-		if (cptr->user->servp
-		    && cptr->user->servp->userlist == cptr->user)
-			if (cptr->user->nextu
-			    && cptr->user->nextu->servp == cptr->user->servp)
-				cptr->user->servp->userlist =cptr->user->nextu;
-			else
-				cptr->user->servp->userlist = NULL;
-		if (cptr->user->nextu)
-			cptr->user->nextu->prevu = cptr->user->prevu;
-		if (cptr->user->prevu)
-			cptr->user->prevu->nextu = cptr->user->nextu;
-		if (usrtop == cptr->user)
-		    {
-			usrtop = cptr->user->nextu;
-			usrtop->prevu = NULL;
-		    }
-		cptr->user->prevu = NULL;
-		cptr->user->nextu = NULL;
-		
 		/* decrement reference counter, and eventually free it */
 		cptr->user->bcptr = NULL;
 		(void)free_user(cptr->user, cptr);
@@ -390,17 +347,6 @@ Reg	aClient	*cptr;
 
 	if (cptr->serv)
 	    {
-		if (cptr->serv->userlist)
-#ifdef DEBUGMODE
-			dumpcore("%#x server %s %#x",
-				 cptr, cptr ? cptr->name : "<noname>",
-				 cptr->serv->userlist);
-#else
-			sendto_flag(SCH_ERROR, "* %#x server %s %#x *",
-				    cptr, cptr ? cptr->name : "<noname>",
-				    cptr->serv->userlist);
-#endif
-
 		/* has to be removed from the list of aServer structures */
 		if (cptr->serv->nexts)
 			cptr->serv->nexts->prevs = cptr->serv->prevs;
@@ -442,6 +388,48 @@ Reg	aClient	*cptr;
 	return;
 }
 
+/*
+ * move the client aClient struct before its server's
+ */
+void	reorder_client_in_list(cptr)
+aClient	*cptr;
+{
+    if (cptr->user == NULL && cptr->service == NULL)
+	    return;
+
+    /* update neighbours */
+    if (cptr->next)
+	    cptr->next->prev = cptr->prev;
+    if (cptr->prev)
+	    cptr->prev->next = cptr->next;
+    else
+	    client = cptr->next;
+
+    /* re-insert */
+    if (cptr->user)
+	{
+	    cptr->next = cptr->user->servp->bcptr;
+	    cptr->prev = cptr->user->servp->bcptr->prev;
+#ifdef DEBUGMODE
+	    sendto_flag(SCH_DEBUG, "%p [%s] moved before server: %p [%s]", 
+			cptr, cptr->name, cptr->user->servp->bcptr, 
+			cptr->user->servp->bcptr->name);
+#endif
+	}
+    else if (cptr->service)
+	{
+	    cptr->next = cptr->service->servp->bcptr;
+	    cptr->prev = cptr->service->servp->bcptr->prev;
+	}
+
+    /* update new neighbours */
+    if (cptr->prev)
+	    cptr->prev->next = cptr;
+    else
+	    client = cptr;
+    cptr->next->prev = cptr;
+}
+
 /*
  * although only a small routine, it appears in a number of places
  * as a collection of a few lines...functions like this *should* be
@@ -461,10 +449,13 @@ aClient	*cptr;
 		istat.is_remc++;
 	if (cptr->user)
 		istat.is_users++;
+	
 	cptr->next = client;
 	client = cptr;
+	
 	if (cptr->next)
 		cptr->next->prev = cptr;
+
 	numclients++;
 	return;
 }
