@@ -24,7 +24,7 @@
 #undef RES_C
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: res.c,v 1.18 1998/12/13 00:19:03 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: res.c,v 1.21.2.5 2000/05/29 19:34:14 q Exp $";
 #endif
 
 /* #undef	DEBUG	/* because there is a lot of debug code in here :-) */
@@ -249,8 +249,6 @@ time_t	now;
 					sendto_iauth("%d d", cptr->fd);
 #endif
 					ClearDNS(cptr);
-					if (!DoingAuth(cptr))	/* XAuth? */
-						SetAccess(cptr); /* unused */
 					break;
 				case ASYNC_CONNECT :
 					sendto_flag(SCH_ERROR,
@@ -589,12 +587,7 @@ HEADER	*hptr;
 	cp = buf + sizeof(HEADER);
 	hp = (struct hent *)&(rptr->he);
 	adr = &hp->h_addr;
-#ifdef INET6
-	while (adr->s6_laddr[0] | adr->s6_laddr[1] | adr->s6_laddr[2] |
-	       adr->s6_laddr[3])
-#else
-	while (adr->s_addr)
-#endif
+	while (WHOSTENTP(adr->S_ADDR))
 		adr++;
 	alias = hp->h_aliases;
 	while (*alias)
@@ -619,13 +612,13 @@ HEADER	*hptr;
 
 		cp += n;
 		type = (int)ircd_getshort((u_char *)cp);
-		cp += sizeof(short);
+		cp += 2; /* INT16SZ */
 		class = (int)ircd_getshort((u_char *)cp);
-		cp += sizeof(short);
+		cp += 2; /* INT16SZ */
 		rptr->ttl = ircd_getlong((u_char *)cp);
-		cp += sizeof(rptr->ttl);
+		cp += 4; /* INT32SZ */
 		dlen =  (int)ircd_getshort((u_char *)cp);
-		cp += sizeof(short);
+		cp += 2; /* INT16SZ */
 		rptr->type = type;
 
 		len = strlen(hostbuf);
@@ -669,9 +662,10 @@ HEADER	*hptr;
 			if (type == T_AAAA)
 				bcopy(cp, (char *)&dr, dlen);
 			else {
-				dr.s6_laddr[0]=dr.s6_laddr[1]=0;
-				dr.s6_laddr[2]=htonl(0xffff);
-				bcopy(cp, &dr.s6_laddr[3], INADDRSZ);
+				/* ugly hack */
+				memset(dr.s6_addr, 0, 10);
+				dr.s6_addr[10] = dr.s6_addr[11] = 0xff;
+				memcpy(dr.s6_addr+12, cp, 4);
 			}
 			bcopy(dr.s6_addr, adr->s6_addr, IN6ADDRSZ);
 #else
@@ -688,7 +682,7 @@ HEADER	*hptr;
 			       inetntoa((char *)adr),
 			       hostbuf));
 #endif
-			if (!hp->h_name)
+			if (!hp->h_name && len < HOSTLEN)
 			    {
 				hp->h_name =(char *)MyMalloc(len+1);
 				(void)strcpy(hp->h_name, hostbuf);
@@ -707,6 +701,10 @@ HEADER	*hptr;
 			    }
 			cp += n;
 			len = strlen(hostbuf);
+			if (len > HOSTLEN)
+			    {
+				return -1;
+			    }
 			Debug((DEBUG_INFO, "got host %s (%d vs %d)",
 				hostbuf, len, strlen(hostbuf)));
 			if (bad_hostname(hostbuf, len))
@@ -718,11 +716,7 @@ HEADER	*hptr;
 			 */
 			if (hp->h_name)
 			    {
-				if (alias >= &(hp->h_aliases[MAXALIASES-1]))
-					break;
-				*alias = (char *)MyMalloc(len + 1);
-				(void)strcpy(*alias++, hostbuf);
-				*alias = NULL;
+				Debug((DEBUG_INFO, "duplicate PTR ignored"));
 			    }
 			else
 			    {
@@ -733,6 +727,10 @@ HEADER	*hptr;
 			break;
 		case T_CNAME :
 			cp += dlen;
+			if (len > HOSTLEN)
+			    {
+				return -1;
+			    }
 			Debug((DEBUG_INFO,"got cname %s",hostbuf));
 			if (bad_hostname(hostbuf, len))
 				return -1; /* a break would be enough here */
@@ -1057,17 +1055,14 @@ Reg	aCache	*ocp;
 	hashtable[hashv].num_list = ocp;
 
 #ifdef	DEBUG
-#ifdef INET6
-	Debug((DEBUG_INFO,"add_to_cache:added %s[%08x%08x%08x%08x] cache %#x.",
-	       ocp->he.h_name,
-	       ((struct in6_addr *)ocp->he.h_addr_list)->s6_laddr[0],
-	       ((struct in6_addr *)ocp->he.h_addr_list)->s6_laddr[1],
-	       ((struct in6_addr *)ocp->he.h_addr_list)->s6_laddr[2],
-	       ((struct in6_addr *)ocp->he.h_addr_list)->s6_laddr[3], ocp));
-#else
+# ifdef	INET6
+	inetntop(AF_INET6, ocp->he.h_addr_list, mydummy, sizeof(mydummy));
+	Debug((DEBUG_INFO,"add_to_cache:added %s[%s] cache %#x.",
+		ocp->he.h_name, mydummy, ocp));
+# else
 	Debug((DEBUG_INFO, "add_to_cache:added %s[%08x] cache %#x.",
 		ocp->he.h_name, ocp->he.h_addr_list[0], ocp));
-#endif
+# endif
 	Debug((DEBUG_INFO,
 		"add_to_cache:h1 %d h2 %x lnext %#x namnext %#x numnext %#x",
 		hash_name(ocp->he.h_name), hashv, ocp->list_next,
@@ -1165,17 +1160,13 @@ aCache	*cachep;
 	/*
 	 * Do the same again for IP#'s.
 	 */
-#ifdef INET6
-	for (s = (char *)rptr->he.h_addr.S_ADDR;
-	     ((struct IN_ADDR *)s)->S_ADDR; s += sizeof(struct IN_ADDR))
-#else
-	for (s = (char *)&rptr->he.h_addr.S_ADDR;
-	     ((struct IN_ADDR *)s)->S_ADDR; s += sizeof(struct IN_ADDR))
-#endif
+	for (j = 0; WHOSTENTP(rptr->he.h_addr_list[j].S_ADDR); j++)
 	    {
 #ifdef INET6
+		s = (char *)rptr->he.h_addr_list[j].S_ADDR;
 		for (i = 0; (t = cp->he.h_addr_list[i]); i++)
 #else
+		s = (char *)&rptr->he.h_addr_list[j].S_ADDR;
 		for (i = 0; (t = cp->he.h_addr_list[i]); i++)
 #endif
 			if (!bcmp(s, t, sizeof(struct IN_ADDR)))
@@ -1202,12 +1193,19 @@ aCache	*cachep;
 					(addrcount + 1) * sizeof(*ab));
 			cp->he.h_addr_list = base;
 #ifdef	DEBUG
+# ifdef	INET6
+			Debug((DEBUG_DNS,"u_l:add IP %s hal %x ac %d",
+				inetntop(AF_INET6, 
+				(char *)((struct in6_addr *)s)->s6_addr,
+				mydummy, MYDUMMY_SIZE),
+				cp->he.h_addr_list, addrcount));
+# else
 			Debug((DEBUG_DNS,"u_l:add IP %x hal %x ac %d",
-				ntohl(((struct IN_ADDR *)s)->S_ADDR),
-				cp->he.h_addr_list,
-				addrcount));
+				ntohl(((struct in_addr *)s)->s_addr),
+				cp->he.h_addr_list, addrcount));
+# endif
 #endif
-			for (; addrcount; addrcount--)
+			for (i = addrcount; i; i--)
 			    {
 				*ab++ = (struct IN_ADDR *)t;
 				t += sizeof(struct IN_ADDR);
@@ -1271,23 +1269,19 @@ char	*numb;
 {
 	Reg	aCache	*cp;
 	Reg	int	hashv,i;
-#ifdef	DEBUG
-	struct	IN_ADDR	*ip = (struct IN_ADDR *)numb;
-#endif
 
 	hashv = hash_number((u_char *)numb);
 
 	cp = hashtable[hashv].num_list;
-#ifdef DEBUG
-#ifdef INET6
-	Debug((DEBUG_DNS,
-	       "find_cache_number:find %s[%08x%08x%08x%08x]: hashv = %d",
-	       inet_ntop(AF_INET6, numb,mydummy,MYDUMMY_SIZE), ip->s6_laddr[0],
-	       ip->s6_laddr[1], ip->s6_laddr[2], ip->s6_laddr[3], hashv));
-#else
+#ifdef	DEBUG
+# ifdef	INET6
+	Debug((DEBUG_DNS, "find_cache_number:find %s: hashv = %d",
+		inet_ntop(AF_INET6, numb, mydummy, MYDUMMY_SIZE), hashv));
+# else
 	Debug((DEBUG_DNS,"find_cache_number:find %s[%08x]: hashv = %d",
-		inetntoa(numb), ntohl(ip->s_addr), hashv));
-#endif
+		inetntoa(numb),	ntohl(((struct in_addr *)numb)->s_addr), 
+		hashv));
+# endif
 #endif
 	for (; cp; cp = cp->hnum_next) 
 	    {
