@@ -24,22 +24,29 @@
 /*  External definitions for global functions.
  */
 #ifndef S_AUTH_C
-#define EXTERN extern
-#else /* S_AUTH_C */
-#define EXTERN
-#endif /* S_AUTH_C */
 # if defined(USE_IAUTH)
-#  if ! USE_STDARG
+extern u_char iauth_options;
+extern u_int iauth_spawn;
+# endif
+#
+# define EXTERN extern
+#else /* S_AUTH_C */
+# define EXTERN
+#endif /* S_AUTH_C */
+
+#if defined(USE_IAUTH)
+# if ! USE_STDARG
 EXTERN int sendto_iauth();
-#  else /* USE_STDARG */
+# else /* USE_STDARG */
 EXTERN int vsendto_iauth (char *pattern, va_list va);
 EXTERN int sendto_iauth (char *pattern, ...);
-#  endif
+# endif
 EXTERN void read_iauth();
 EXTERN void report_iauth_conf __P((aClient *, char *));
 EXTERN void report_iauth_stats __P((aClient *, char *));
-# endif
+#endif
 EXTERN void start_auth __P((Reg aClient *cptr));
 EXTERN void send_authports __P((aClient *cptr));
 EXTERN void read_authports __P((Reg aClient *cptr));
+
 #undef EXTERN
