@@ -35,7 +35,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: s_bsd.c,v 1.52 1999/02/03 22:12:34 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: s_bsd.c,v 1.73.2.19 2001/06/30 20:18:55 q Exp $";
 #endif
 
 #include "os.h"
@@ -132,7 +132,7 @@ int	size;
 **	text	is a *format* string for outputting error. It must
 **		contain only two '%s', the first will be replaced
 **		by the sockhost from the cptr, and the latter will
-**		be taken from sys_errlist[errno].
+**		by strerror(errno).
 **
 **	cptr	if not NULL, is the *LOCAL* client associated with
 **		the error.
@@ -157,7 +157,7 @@ aClient *cptr;
 	 * gamble anyway.
 	 */
 #ifdef	SO_ERROR
-	if (!IsMe(cptr) && cptr->fd >= 0)
+	if (cptr && !IsMe(cptr) && cptr->fd >= 0)
 		if (!GETSOCKOPT(cptr->fd, SOL_SOCKET, SO_ERROR, &err, &len))
 			if (err)
 				errtmp = err;
@@ -192,7 +192,7 @@ int	port;
 	/*
 	 * do it this way because building ip# from separate values for each
 	 * byte requires endian knowledge or some nasty messing. Also means
-	 * easy conversion of "*" 0.0.0.0 or 134.* to 134.0.0.0 :-)
+	 * easy conversion of "*" to 0.0.0.0 or 134.* to 134.0.0.0 :-)
 	 */
 	(void)sscanf(ipmask, "%d.%d.%d.%d", &ad[0], &ad[1], &ad[2], &ad[3]);
 	(void)sprintf(ipname, "%d.%d.%d.%d", ad[0], ad[1], ad[2], ad[3]);
@@ -226,14 +226,15 @@ int	port;
 	if (port)
 	    {
 		server.SIN_FAMILY = AFINET;
-		if (!ip || !isdigit(*ip))
 #ifdef INET6
+		if (!ip || (!isxdigit(*ip) && *ip != ':'))
 			server.sin6_addr = in6addr_any;
 		else
-			if(!inet_pton(AF_INET6, ip, server.sin6_addr.s6_addr))
+			if(!inetpton(AF_INET6, ip, server.sin6_addr.s6_addr))
 				bcopy(minus_one, server.sin6_addr.s6_addr,
 				      IN6ADDRSZ);
 #else
+		if (!ip || !isdigit(*ip))
 			server.sin_addr.s_addr = INADDR_ANY;
 		else
 			server.sin_addr.s_addr = inetaddr(ip);
@@ -433,27 +434,26 @@ int rcvdsig;
 #if defined(USE_IAUTH)
 	static time_t last = 0;
 	static char first = 1;
-	int sp[2], fd;
+	int sp[2], fd, val;
 
 	if ((bootopt & BOOT_NOIAUTH) != 0)
 		return;
 	if (adfd >= 0)
 	    {
-		sendto_flag(SCH_AUTH,
-			    "iauth is already running, restart aborted");
+		if (rcvdsig)
+			sendto_flag(SCH_AUTH,
+			    "iauth is already running, restart canceled");
 		return;
 	    }
-	read_iauth(); /* to reset olen */
-	if ((time(NULL) - last) > 300 || rcvdsig)
+	if ((time(NULL) - last) > 90 || rcvdsig)
 	    {
 		sendto_flag(SCH_AUTH, "Starting iauth...");
 		last = time(NULL);
+		read_iauth(); /* to reset olen */
+		iauth_spawn += 1;
 	    }
 	else
-	    {
-		sendto_flag(SCH_AUTH, "Not restarting iauth.");
 		return;
-	    }
 	if (socketpair(AF_UNIX, SOCK_STREAM, 0, sp) < 0)
 	    {
 		sendto_flag(SCH_ERROR, "socketpair() failed!");
@@ -462,27 +462,44 @@ int rcvdsig;
 	adfd = sp[0];
 	set_non_blocking(sp[0], NULL);
 	set_non_blocking(sp[1], NULL); /* less to worry about in iauth */
+	val = IAUTH_BUFFER;
+	if (setsockopt(sp[0], SOL_SOCKET, SO_SNDBUF, (void *) &val,
+	    sizeof(val)) < 0)
+			sendto_flag(SCH_AUTH,
+			    "IAUTH_BUFFER too big for sp0 sndbuf, using default");
+	if (setsockopt(sp[1], SOL_SOCKET, SO_SNDBUF, (void *) &val,
+	    sizeof(val)) < 0)
+			sendto_flag(SCH_AUTH,
+			    "IAUTH_BUFFER too big for sp1 sndbuf, using default");
+	if (setsockopt(sp[0], SOL_SOCKET, SO_RCVBUF, (void *) &val,
+	    sizeof(val)) < 0)
+			sendto_flag(SCH_AUTH,
+			    "IAUTH_BUFFER too big for sp0 rcvbuf, using default");
+	if (setsockopt(sp[1], SOL_SOCKET, SO_RCVBUF, (void *) &val,
+	    sizeof(val)) < 0)
+			sendto_flag(SCH_AUTH,
+			    "IAUTH_BUFFER too big for sp1 rcvbuf, using default");
 	switch (vfork())
 	    {
-	case -1:
-		sendto_flag(SCH_ERROR, "vfork() failed!");
-		sendto_flag(SCH_AUTH, "Failed to restart iauth!");
-		close(sp[0]); close(sp[1]);
-		adfd = -1;
-		return;
-	case 0:
-		for (fd = 0; fd < MAXCONNECTIONS; fd++)
-			if (fd != sp[1])
-				(void)close(fd);
-		if (sp[1] != 0)
-		    {
-			(void)dup2(sp[1], 0);
+		case -1:
+			sendto_flag(SCH_ERROR, "vfork() failed!");
+			sendto_flag(SCH_AUTH, "Failed to restart iauth!");
+			close(sp[0]); close(sp[1]);
+			adfd = -1;
+			return;
+		case 0:
+			for (fd = 0; fd < MAXCONNECTIONS; fd++)
+				if (fd != sp[1])
+					(void)close(fd);
+			if (sp[1] != 0)
+			    {
+				(void)dup2(sp[1], 0);
+				close(sp[1]);
+			    }
+			if (execl(IAUTH_PATH, IAUTH, NULL) < 0)
+				_exit(-1); /* should really not happen.. */
+		default:
 			close(sp[1]);
-		    }
-		if (execl(APATH, APATH, NULL) < 0)
-			_exit(-1); /* should really not happen.. */
-	default:
-		close(sp[1]);
 	    }
 
 	if (first)
@@ -615,26 +632,24 @@ init_dgram:
 
 void	write_pidfile()
 {
-#ifdef IRCD_PIDFILE
 	int fd;
 	char buff[20];
-	(void)truncate(IRCD_PIDFILE, 0);
-	if ((fd = open(IRCD_PIDFILE, O_CREAT|O_WRONLY, 0600))>=0)
+	(void)truncate(IRCDPID_PATH, 0);
+	if ((fd = open(IRCDPID_PATH, O_CREAT|O_WRONLY, 0600))>=0)
 	    {
 		bzero(buff, sizeof(buff));
 		(void)sprintf(buff,"%5d\n", (int)getpid());
 		if (write(fd, buff, strlen(buff)) == -1)
 			Debug((DEBUG_NOTICE,"Error writing to pid file %s",
-			      IRCD_PIDFILE));
+			      IRCDPID_PATH));
 		(void)close(fd);
 		return;
 	    }
 # ifdef	DEBUGMODE
 	else
 		Debug((DEBUG_NOTICE,"Error opening pid file %s",
-			IRCD_PIDFILE));
+			IRCDPID_PATH));
 # endif
-#endif
 }
 		
 /*
@@ -682,8 +697,7 @@ Reg	char	*sockn;
 	if (inetnetof(sk.SIN_ADDR) == IN_LOOPBACKNET)
 #endif
 	    {
-		cptr->hostp = NULL;
-		strncpyzt(sockn, me.sockhost, HOSTLEN);
+		cptr->hostp = me.hostp;
 	    }
 	bcopy((char *)&sk.SIN_ADDR, (char *)&cptr->ip, sizeof(struct IN_ADDR));
 	cptr->port = (int)(ntohs(sk.SIN_PORT));
@@ -705,7 +719,6 @@ Reg	aClient	*cptr;
 	Reg	struct	hostent *hp = NULL;
 	Reg	int	i;
  
-	ClearAccess(cptr);
 #ifdef INET6
 	Debug((DEBUG_DNS, "ch_cl: check access for %s[%s]",
 		cptr->name, inet_ntop(AF_INET6, (char *)&cptr->ip, mydummy,
@@ -723,8 +736,10 @@ Reg	aClient	*cptr;
 	/*
 	 * Verify that the host to ip mapping is correct both ways and that
 	 * the ip#(s) for the socket is listed for the host.
+	 * We shouldn't check it for localhost, because hp is fake in that
+	 * case. -Toor
 	 */
-	if (hp)
+	if (hp && (hp != me.hostp))
 	    {
 		for (i = 0; hp->h_addr_list[i]; i++)
 			if (!bcmp(hp->h_addr_list[i], (char *)&cptr->ip,
@@ -762,8 +777,7 @@ Reg	aClient	*cptr;
 
 #ifdef INET6
 	if (IN6_IS_ADDR_LOOPBACK(&cptr->ip) || IsUnixSocket(cptr) ||
-		(cptr->ip.s6_laddr[0]==mysk.sin6_addr.s6_laddr[0] &&
-		 cptr->ip.s6_laddr[1]==mysk.sin6_addr.s6_laddr[1])
+	    !memcmp(cptr->ip.s6_addr, mysk.sin6_addr.s6_addr, 8) 
 /* ||
 	    IN6_ARE_ADDR_SAMEPREFIX(&cptr->ip, &mysk.SIN_ADDR))
  about the same, I think              NOT */
@@ -858,7 +872,6 @@ aClient	*cptr;
 			** get us here and we can't interrupt that very
 			** well.
 			*/
-			ClearAccess(cptr);
 			lin.value.aconf = aconf;
 			lin.flags = ASYNC_CONF;
 			nextdnscheck = 1;
@@ -885,7 +898,6 @@ int	estab;
 	Link	*lp = cptr->confs;
 	int	i;
 
-	ClearAccess(cptr);
 	if (check_init(cptr, sockname))
 		return -2;
 
@@ -1172,7 +1184,7 @@ void	close_connection(cptr)
 aClient *cptr;
 {
 	Reg	aConfItem *aconf;
-	Reg	int	i,j;
+	Reg	int	i;
 #ifdef SO_LINGER
 	struct 	linger	sockling;
 
@@ -1287,40 +1299,6 @@ aClient *cptr;
 		local[i] = NULL;
 		(void)close(i);
 
-		/*
-		 * fd remap to keep local[i] filled at the bottom.
-		 *	don't *ever* move descriptors for 
-		 *		+ log file
-		 *		+ sockets bound to listen() ports
-		 *	--Yegg
-		 */
-		if (i >= 0 && (j = highest_fd) > i)
-		    {
-			while (!local[j])
-				j--;
-			if (j > i && local[j] &&
-			    !(IsLog(local[j]) || IsMe(local[j])))
-			    {
-				if (dup2(j,i) == -1)
-					return;
-				local[i] = local[j];
-				local[i]->fd = i;
-				local[j] = NULL;
-				(void)close(j);
-				del_fd(j, &fdall);
-				add_fd(i, &fdall);
-				if (IsServer(local[i]) || IsMe(local[i]))
-				    {
-					del_fd(j, &fdas);
-					add_fd(i, &fdas);
-				    }
-				while (!local[highest_fd])
-					highest_fd--;
-#if defined(USE_IAUTH)
-				sendto_iauth("%d R %d", j, i);
-#endif
-			    }
-		    }
 		cptr->fd = -2;
 		DBufClear(&cptr->sendQ);
 		DBufClear(&cptr->recvQ);
@@ -1384,9 +1362,21 @@ aClient	*cptr;
 	opt = 8192;
 	if (SETSOCKOPT(fd, SOL_SOCKET, SO_SNDBUF, &opt, opt) < 0)
 		report_error("setsockopt(SO_SNDBUF) %s:%s", cptr);
+# ifdef	SO_SNDLOWAT
+	/*
+	 * Setting the low water mark should improve performence by avoiding
+	 * early returns from select()/poll().  It shouldn't delay sending
+	 * data, provided that io_loop() combines read_message() and
+	 * flush_fdary/connections() calls properly. -kalt
+	 * This call isn't always implemented, even when defined.. so be quiet
+	 * about errors. -kalt
+	 */
+	opt = 8192;
+	SETSOCKOPT(fd, SOL_SOCKET, SO_SNDLOWAT, &opt, opt);
+# endif
 #endif
 #if defined(IP_OPTIONS) && defined(IPPROTO_IP) && !defined(AIX) && \
-    !defined(SUN_GSO_BUG)
+    !defined(SUN_GSO_BUG) && !defined(INET6)
 	/*
 	 * Mainly to turn off and alert us to source routing, here.
 	 * Method borrowed from Wietse Venema's TCP wrapper.
@@ -1507,7 +1497,11 @@ aClient *cptr;
 	    }
 	/* Now add new item to the list */
 	blptr = (struct abacklog *) MyMalloc(sizeof(struct abacklog));
+#ifdef INET6
+	bcopy(cptr->ip.s6_addr, blptr->ip.s6_addr, IN6ADDRSZ);
+#else
 	blptr->ip.s_addr = cptr->ip.s_addr;
+#endif
 	blptr->PT = timeofday;
 	blptr->next = backlog;
 	backlog = blptr;
@@ -1516,7 +1510,11 @@ aClient *cptr;
 	blptr = backlog;
 	while (blptr != NULL)
 	    {
+#ifdef INET6
+		if (bcmp(blptr->ip.s6_addr, cptr->ip.s6_addr, IN6ADDRSZ) == 0)
+#else
 		if (blptr->ip.s_addr == cptr->ip.s_addr)
+#endif
 			count++;
 		blptr = blptr->next;
 	    }
@@ -1584,6 +1582,7 @@ add_con_refuse:
 
 		lin.flags = ASYNC_CLIENT;
 		lin.value.cptr = acptr;
+		lin.next = NULL;
 #ifdef INET6
 		Debug((DEBUG_DNS, "lookup %s",
 		       inet_ntop(AF_INET6, (char *)&addr.sin6_addr,
@@ -1604,10 +1603,13 @@ add_con_refuse:
 		sendto_flag(SCH_LOCAL, "Rejecting connection from %s[%s].",
 			    (acptr->hostp) ? acptr->hostp->h_name : "",
 			    acptr->sockhost);
-		sendto_flog(acptr, " ?Clone? ", 0, "<none>",
+		sendto_flog(acptr, " ?Clone? ", "<none>",
 			    (acptr->hostp) ? acptr->hostp->h_name :
 			    acptr->sockhost);
 		del_queries((char *)acptr);
+		(void)send(fd,
+			   "ERROR :Too rapid connections from your host\r\n",
+			   46, 0);
 		goto add_con_refuse;
 	    }
 #endif
@@ -1634,6 +1636,8 @@ add_con_refuse:
 				     acptr->hostp->h_aliases[i++]);
 		if (acptr->hostp->h_name)
 			sendto_iauth("%d N %s",acptr->fd,acptr->hostp->h_name);
+		else if (acptr->hostp->h_aliases[0])
+			sendto_iauth("%d n", acptr->fd);
 	    }
 #endif
 	return acptr;
@@ -1679,11 +1683,80 @@ int	fd;
 	add_client_to_list(acptr);
 	set_non_blocking(acptr->fd, acptr);
 	(void)set_sock_opts(acptr->fd, acptr);
-	SetAccess(acptr);
+# if defined(USE_IAUTH)
+	/*
+	** iauth protocol and iauth itself should be extended to alllow
+	** dealing with this type of connection.
+	*/
+	sendto_iauth("%d O", acptr->fd);
+	SetDoneXAuth(acptr);
+# endif
 	return;
 }
 #endif
 
+/*
+** read_listener
+**
+** Accept incoming connections, extracted from read_message() 98/12 -kalt
+** Up to 10 connections will be accepted, unless SLOW_ACCEPT is defined.
+*/
+static void
+read_listener(cptr)
+aClient *cptr;
+{
+	int fdnew, max = 10;
+
+#if defined(SLOW_ACCEPT)
+	max = 1;
+#endif
+	while (max--)
+	    {
+		/*
+		** There may be many reasons for error return, but in otherwise
+		** correctly working environment the probable cause is running
+		** out of file descriptors (EMFILE, ENFILE or others?). The
+		** man pages for accept don't seem to list these as possible,
+		** although it's obvious that it may happen here.
+		** Thus no specific errors are tested at this point, just
+		** assume that connections cannot be accepted until some old
+		** is closed first.
+		*/
+		if ((fdnew = accept(cptr->fd, NULL, NULL)) < 0)
+		    {
+			if (errno != EWOULDBLOCK)
+				report_error("Cannot accept connection %s:%s",
+					     cptr);
+			break;
+		    }
+		ircstp->is_ac++;
+		if (fdnew >= MAXCLIENTS)
+		    {
+			ircstp->is_ref++;
+			sendto_flag(SCH_ERROR, "All connections in use. (%s)",
+				    get_client_name(cptr, TRUE));
+			find_bounce(NULL, 0, fdnew);
+			(void)send(fdnew, "ERROR :All connections in use\r\n",
+				   32, 0);
+			(void)close(fdnew);
+			continue;
+		    }
+		/*
+		 * Use of add_connection (which never fails :) meLazy
+		 * Never say never. MrMurphy visited here. -Vesa
+		 */
+#ifdef	UNIXPORT
+		if (IsUnixSocket(cptr))
+			add_unixconnection(cptr, fdnew);
+		else
+#endif
+			if (!add_connection(cptr, fdnew))
+				continue;
+		nextping = timeofday; /* isn't this abusive? -kalt */
+		istat.is_unknown++;
+	    }
+}
+
 /*
 ** client_packet
 **
@@ -1811,21 +1884,6 @@ int	msg_ready;
 			done = dopacket(cptr, readbuf, length);
 			if (done && done != 2)
 				return done;
-#ifdef ZIP_LINKS
-			/*
-			** compressed data received cannot always be
-			** entirely uncompressed in one step (because of
-			** buffer sizes); this makes sure to uncompress
-			** all that was received.
-			*/
-			if (cptr->flags & FLAGS_ZIP)
-				while (cptr->zip->in->avail_in != 0)
-				    {
-					done = dopacket(cptr, readbuf, 0);
-					if (done && done != 2)
-						return done;
-				    }
-#endif
 		    }
 	    }
 	else
@@ -1856,11 +1914,13 @@ int	msg_ready;
  * processed. Also check for connections with data queued and whether we can
  * write it out.
  */
-int	read_message(delay, fdp)
+int	read_message(delay, fdp, ro)
 time_t	delay; /* Don't ever use ZERO here, unless you mean to poll and then
 		* you have to have sleep/wait somewhere else in the code.--msa
+		* Actually, ZERO is NOT ZERO anymore.. see below -kalt
 		*/
 FdAry	*fdp;
+int	ro;
 {
 #if ! USE_POLL
 # define SET_READ_EVENT( thisfd )	FD_SET( thisfd, &read_set)
@@ -1905,12 +1965,11 @@ FdAry	*fdp;
 	int	   nbr_pfds = 0;
 #endif
 
-	Reg	aClient	*cptr;
-	Reg	int	nfds;
+	aClient	*cptr;
+	int	nfds, ret = 0;
 	struct	timeval	wait;
 	time_t	delay2 = delay;
-	u_long	usec = 0;
-	int	res, length, fd, i, fdnew;
+	int	res, length, fd, i;
 	int	auth;
 
 	for (res = 0;;)
@@ -1941,6 +2000,7 @@ FdAry	*fdp;
 			Debug((DEBUG_L11, "fd %d cptr %#x %d %#x %s",
 				fd, cptr, cptr->status, cptr->flags,
 				get_client_name(cptr,TRUE)));
+			/* authentication fd's */
 			if (DoingAuth(cptr))
 			    {
 				auth++;
@@ -1956,48 +2016,66 @@ FdAry	*fdp;
 					highfd = cptr->authfd;
 #endif
 			    }
+			/*
+			** if any of these is true, data won't be parsed
+			** so no need to check for anything!
+			*/
+#if defined(USE_IAUTH)
 			if (DoingDNS(cptr) || DoingAuth(cptr) ||
-			    DoingXAuth(cptr))
+			    WaitingXAuth(cptr) ||
+			    (DoingXAuth(cptr) &&
+			     !(iauth_options & XOPT_EARLYPARSE)))
+#else
+			if (DoingDNS(cptr) || DoingAuth(cptr))
+#endif
 				continue;
 #if ! USE_POLL
 			if (fd > highfd)
 				highfd = fd;
 #endif
+			/*
+			** Checking for new connections is only done up to
+			** once per second.
+			*/
 			if (IsListening(cptr))
 			    {
-#ifndef	SLOW_ACCEPT
-					if (IsUnixSocket(cptr))
-					    {
-#endif
-						if ((timeofday > cptr->lasttime + 2))
-					    {
-						SET_READ_EVENT( fd );
-					    }
-					else if (delay2 > 2)
-						delay2 = 2;
-#ifndef	SLOW_ACCEPT
+				if (timeofday > cptr->lasttime + 1 && ro == 0)
+				    {
+					SET_READ_EVENT( fd );
 				    }
-				else
-					SET_READ_EVENT( fd );
-#endif
-			    }
-			else
-			    {
-				if (DBufLength(&cptr->recvQ) && delay2 > 2)
+				else if (delay2 > 1)
 					delay2 = 1;
-				if (DBufLength(&cptr->recvQ) < 4088)
-					SET_READ_EVENT( fd );
+				continue;
 			    }
 			
+			/*
+			** This is very approximate, it should take
+			** cptr->since into account. -kalt
+			*/
+			if (DBufLength(&cptr->recvQ) && delay2 > 2)
+				delay2 = 1;
+
+			if (IsRegisteredUser(cptr))
+			    {
+				if (cptr->since - timeofday < MAXPENALTY+1)
+					SET_READ_EVENT( fd );
+			    }
+			else if (DBufLength(&cptr->recvQ) < 4088)
+				SET_READ_EVENT( fd );
+
+			/*
+			** If we have anything in the sendQ, check if there is
+			** room to write data.
+			*/
 			if (DBufLength(&cptr->sendQ) || IsConnecting(cptr) ||
 #ifdef	ZIP_LINKS
-			    IsReconnect(cptr) || ((cptr->flags & FLAGS_ZIP) &&
-						  (cptr->zip->outcount > 0))
-#else
-			    IsReconnect(cptr)
+			    ((cptr->flags & FLAGS_ZIP) &&
+			     (cptr->zip->outcount > 0)) ||
 #endif
-			    ) /* for emacs auto-indentation */
-				SET_WRITE_EVENT( fd );
+			    IsReconnect(cptr))
+				if (IsServer(cptr) || IsConnecting(cptr) ||
+				    ro == 0)
+					SET_WRITE_EVENT( fd );
 		    }
 
 		if (udpfd >= 0)
@@ -2032,14 +2110,14 @@ FdAry	*fdp;
 # endif			
 		    }
 #endif
-		Debug((DEBUG_L11, "udpfd %d resfd %d adfd %s", udpfd, resfd,
+		Debug((DEBUG_L11, "udpfd %d resfd %d adfd %d", udpfd, resfd,
 		       adfd));
 #if ! USE_POLL
 		Debug((DEBUG_L11, "highfd %d", highfd));
 #endif
 		
 		wait.tv_sec = MIN(delay2, delay);
-		wait.tv_usec = usec;
+		wait.tv_usec = (delay == 0) ? 200000 : 0;
 #if ! USE_POLL
 		nfds = select(highfd + 1, (SELECT_FDSET_TYPE *)&read_set,
 			      (SELECT_FDSET_TYPE *)&write_set, 0, &wait);
@@ -2047,6 +2125,7 @@ FdAry	*fdp;
 		nfds = poll( poll_fdarray, nbr_pfds,
 			     wait.tv_sec * 1000 + wait.tv_usec/1000 );
 #endif
+		ret = nfds;
 		if (nfds == -1 && errno == EINTR)
 			return -1;
 		else if (nfds >= 0)
@@ -2058,11 +2137,12 @@ FdAry	*fdp;
 #endif
 		res++;
 		if (res > 5)
-			restart("too many select errors");
+			restart("too many select()/poll() errors");
 		sleep(10);
 		timeofday = time(NULL);
 	    } /* for(res=0;;) */
 	
+	timeofday = time(NULL);
 	if (nfds > 0 &&
 #if ! USE_POLL
 	    resfd >= 0 &&
@@ -2071,9 +2151,9 @@ FdAry	*fdp;
 #endif
 	    TST_READ_EVENT(resfd))
 	    {
-		do_dns_async();
-		nfds--;
 		CLR_READ_EVENT(resfd);
+		nfds--;
+		do_dns_async();
 	    }
 	if (nfds > 0 &&
 #if ! USE_POLL
@@ -2083,9 +2163,9 @@ FdAry	*fdp;
 #endif
 	    TST_READ_EVENT(udpfd))
 	    {
-		polludp();
-		nfds--;
 		CLR_READ_EVENT(udpfd);
+		nfds--;
+		polludp();
 	    }
 #if defined(USE_IAUTH)
 	if (nfds > 0 &&
@@ -2096,9 +2176,9 @@ FdAry	*fdp;
 # endif
 	    TST_READ_EVENT(adfd))
 	    {
-		read_iauth();
-		nfds--;
 		CLR_READ_EVENT(adfd);
+		nfds--;
+		read_iauth();
 	    }
 #endif
 
@@ -2153,56 +2233,7 @@ FdAry	*fdp;
 		    {
 			CLR_READ_EVENT(fd);
 			cptr->lasttime = timeofday;
-			/*
-			** There may be many reasons for error return, but
-			** in otherwise correctly working environment the
-			** probable cause is running out of file descriptors
-			** (EMFILE, ENFILE or others?). The man pages for
-			** accept don't seem to list these as possible,
-			** although it's obvious that it may happen here.
-			** Thus no specific errors are tested at this
-			** point, just assume that connections cannot
-			** be accepted until some old is closed first.
-			*/
-			if ((fdnew = accept(fd, NULL, NULL)) < 0)
-			    {
-				report_error("Cannot accept connections %s:%s",
-					     cptr);
-				continue;
-			    }
-			ircstp->is_ac++;
-			if (fdnew >= MAXCLIENTS)
-			    {
-				ircstp->is_ref++;
-				sendto_flag(SCH_ERROR,
-					    "All connections in use. (%s)",
-					    get_client_name(cptr, TRUE));
-				find_bounce(NULL, 0, fdnew);
-#ifdef INET6
-				(void)sendto(fdnew,
-					   "ERROR :All connections in use\r\n",
-					   32, 0, 0, 0);
-#else
-				(void)send(fdnew,
-					   "ERROR :All connections in use\r\n",
-					   32, 0);
-#endif
-				(void)close(fdnew);
-				continue;
-			    }
-			/*
-			 * Use of add_connection (which never fails :) meLazy
-			 * Never say never. MrMurphy visited here. -Vesa
-			 */
-#ifdef	UNIXPORT
-			if (IsUnixSocket(cptr))
-				add_unixconnection(cptr, fdnew);
-			else
-#endif
-				if (!add_connection(cptr, fdnew))
-					continue;
-			nextping = timeofday;
-			istat.is_unknown++;
+			read_listener(cptr);
 			continue;
 		    }
 		if (IsMe(cptr))
@@ -2222,22 +2253,26 @@ FdAry	*fdp;
 deadsocket:
 				if (TST_READ_EVENT(fd))
 					CLR_READ_EVENT(fd);
-				if (cptr->exitc =! EXITC_UNDEF)
-					sendto_flag(SCH_DEBUG,
-						    "EXITC overwritten: %c",
-						    cptr->exitc);
-				cptr->exitc = EXITC_ERROR;
-				(void)exit_client(cptr, cptr, &me,
-						  strerror(get_sockerr(cptr)));
+				if (cptr->exitc == EXITC_SENDQ)
+				{
+					(void)exit_client(cptr,cptr,&me,
+						"Max SendQ exceeded");
+				}
+				else
+				{
+					cptr->exitc = EXITC_ERROR;
+					(void)exit_client(cptr, cptr, &me,
+						strerror(get_sockerr(cptr)));
+				}
 				continue;
 			    }
 		    }
 		length = 1;	/* for fall through case */
-		if (!NoNewLine(cptr) || TST_READ_EVENT(fd)) {
-		    if (!DoingAuth(cptr))
-			    length = read_packet(cptr,
-						 TST_READ_EVENT(fd));
-		}
+		if (!NoNewLine(cptr) || TST_READ_EVENT(fd))
+		    {
+			if (!DoingAuth(cptr))
+				length = read_packet(cptr, TST_READ_EVENT(fd));
+		    }
 		readcalls++;
 		if (length == FLUSH_BUFFER)
 			continue;
@@ -2301,7 +2336,7 @@ deadsocket:
 				  "EOF From client" :
 				  strerror(get_sockerr(cptr)));
 	    } /* for(i) */
-	return 0;
+	return ret;
 }
 
 /*
@@ -2354,7 +2389,7 @@ struct	hostent	*hp;
 		s = (char *)index(aconf->host, '@');
 		s++; /* should NEVER be NULL */
 #ifdef INET6
-		if (!inet_pton(AF_INET6, s, aconf->ipnum.s6_addr))
+		if (!inetpton(AF_INET6, s, aconf->ipnum.s6_addr))
 #else
 		if ((aconf->ipnum.s_addr = inetaddr(s)) == -1)
 #endif
@@ -2525,7 +2560,7 @@ int	*lenp;
 	 */
 #ifdef INET6
 	if (isdigit(*aconf->host) && (AND16(aconf->ipnum.s6_addr) == 255))
-		if (!inet_pton(AF_INET6, aconf->host,aconf->ipnum.s6_addr))
+		if (!inetpton(AF_INET6, aconf->host,aconf->ipnum.s6_addr))
 			bcopy(minus_one, aconf->ipnum.s6_addr, IN6ADDRSZ);
 	if (AND16(aconf->ipnum.s6_addr) == 255)
 #else
@@ -2651,7 +2686,7 @@ char	*namebuf, *linebuf, *chname;
 {
 	static	char	wrerr[] = "NOTICE %s :Write error. Couldn't summon.";
 	int	fd;
-	char	line[120];
+	char	line[512];
 	struct	tm	*tp;
 
 	tp = localtime(&timeofday);
@@ -2670,6 +2705,7 @@ char	*namebuf, *linebuf, *chname;
 	 */
 	if ((linebuf[0] != 't' || linebuf[1] != 't' || linebuf[2] != 'y')
 	    && (linebuf[0] != 'c' || linebuf[1] != 'o' || linebuf[2] != 'n')
+	    && (linebuf[0] != 'p' || linebuf[1] != 't' || linebuf[2] != 's')
 #ifdef HPUX
 	    && (linebuf[0] != 'p' || linebuf[1] != 't' || linebuf[2] != 'y' ||
 		linebuf[3] != '/')
@@ -2769,6 +2805,10 @@ int	len;
 	struct	hostent	*hp;
 	char	*cname = cptr->name;
 	aConfItem	*aconf;
+#ifdef HAVE_GETIPNODEBYNAME
+	int	error_num1, error_num2;
+	struct	hostent	*hp1, *hp2;
+#endif
 
 	/*
 	** Setup local socket structure to use for binding to.
@@ -2778,7 +2818,7 @@ int	len;
 	
 	if ((aconf = find_me())->passwd && isdigit(*aconf->passwd))
 #ifdef INET6
-		if(!inet_pton(AF_INET6, aconf->passwd, mysk.sin6_addr.s6_addr))
+		if(!inetpton(AF_INET6, aconf->passwd, mysk.sin6_addr.s6_addr))
 			bcopy(minus_one, mysk.sin6_addr.s6_addr, IN6ADDRSZ);
 #else
 		mysk.sin_addr.s_addr = inetaddr(aconf->passwd);
@@ -2799,7 +2839,14 @@ int	len;
 	*/
 	if (BadPtr(cname))
 		return;
+#ifdef HAVE_GETIPNODEBYNAME
+	hp1 = getipnodebyname(cname, AF_INET6, AI_DEFAULT, &error_num1);
+	hp2 = getipnodebyname(name, AF_INET6, AI_DEFAULT, &error_num2);
+	if (! error_num1) hp=hp1; else hp=hp2;
+	if ((! error_num1) || (! error_num2)) 
+#else
 	if ((hp = gethostbyname(cname)) || (hp = gethostbyname(name)))
+#endif
 	    {
 		char	*hname;
 		int	i = 0;
@@ -2828,6 +2875,10 @@ int	len;
 		Debug((DEBUG_DEBUG,"local name is %s",
 				get_client_name(&me,TRUE)));
 	    }
+#ifdef HAVE_GETIPNODEBYNAME
+	freehostent(hp1);
+	freehostent(hp2);
+#endif
 	return;
 }
 
@@ -2845,8 +2896,10 @@ aConfItem	*aconf;
 	bzero((char *)&from, sizeof(from));
 	if (aconf->passwd && isdigit(*aconf->passwd))
 #ifdef INET6
-	  if(!inet_pton(AF_INET6, aconf->passwd,from.sin6_addr.s6_addr))
-		bcopy(minus_one, from.sin6_addr.s6_addr, IN6ADDRSZ);
+	    {
+		if (!inetpton(AF_INET6, aconf->passwd,from.sin6_addr.s6_addr))
+			bcopy(minus_one, from.sin6_addr.s6_addr, IN6ADDRSZ);
+	    }
 #else
 	  from.sin_addr.s_addr = inetaddr(aconf->passwd);
 #endif
@@ -3033,7 +3086,16 @@ static	void	polludp()
 			return;
 		else
 		    {
-			report_error("udp port recvfrom (%s): %s", &me);
+			char buf[100];
+
+			sprintf(buf, "udp port recvfrom() from %s to %%s: %%s",
+#ifdef INET6
+				inetntop(AF_INET6, (char *)&from.sin6_addr, mydummy, MYDUMMY_SIZE)
+#else
+				inetntoa((char *)&from.sin_addr)
+#endif
+				);
+			report_error(buf, &me);
 			return;
 		    }
 	    }
@@ -3143,8 +3205,6 @@ static	void	do_dns_async()
 			    {
 				del_queries((char *)cptr);
 				ClearDNS(cptr);
-				if (!DoingAuth(cptr))
-					SetAccess(cptr);
 				cptr->hostp = hp;
 #if defined(USE_IAUTH)
 				if (hp)
@@ -3158,7 +3218,13 @@ static	void	do_dns_async()
 					if (hp->h_name)
 						sendto_iauth("%d N %s",
 							cptr->fd, hp->h_name);
+					else if (hp->h_aliases[0])
+						sendto_iauth("%d n", cptr->fd);
 				    }
+				else
+					sendto_iauth("%d d", cptr->fd);
+				if (iauth_options & XOPT_EXTWAIT)
+					cptr->lasttime = timeofday;
 #endif
 			    }
 			break;
