@@ -48,7 +48,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: s_conf.c,v 1.36 1998/12/13 00:02:37 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: s_conf.c,v 1.42.2.4 2000/05/29 19:38:25 q Exp $";
 #endif
 
 #include "os.h"
@@ -61,6 +61,7 @@ static	int	check_time_interval __P((char *, char *));
 static	int	lookup_confhost __P((aConfItem *));
 
 aConfItem	*conf = NULL;
+aConfItem	*kconf = NULL;
 
 /*
  * remove all conf entries from the client except those which match
@@ -82,30 +83,68 @@ int	mask;
 
 /*
  * Match address by #IP bitmask (10.11.12.128/27)
+ * Now should work for IPv6 too.
+ * returns -1 on error, 0 on match, 1 when NO match.
  */
 int    match_ipmask(mask, cptr)
 char   *mask;
 aClient *cptr;
 {
-        int i1, i2, i3, i4, m;
-        u_long lmask, baseip;
-	char *at;
+	int	m;
+	char	*p;
+	struct  IN_ADDR addr;
+	char	dummy[128];
+	u_long	lmask;
+#ifdef	INET6
+	int	j;
+#endif;
  
-	if (at = index(mask, '@'))
-		mask = at + 1;
-        if (sscanf(mask, "%d.%d.%d.%d/%d", &i1, &i2, &i3, &i4, &m) != 5 ||
-           m < 1 || m > 31) {
-               sendto_flag(SCH_LOCAL, "Ignoring bad mask: %s", mask);
-                return -1;
-        }
-        lmask = htonl((u_long)0xffffffffL << (32 - m)); /* /24->0xffffff00ul */
-        baseip = htonl(i1 * 0x1000000 + i2 * 0x10000 + i3 * 0x100 + i4);
-#ifdef INET6
-	return 1;
-/*        return ((cptr->ip.s6_addr & lmask) == baseip) ? 0 : 1;*/
+	strncpyzt(dummy, mask, sizeof(dummy));
+	mask = dummy;
+	if ((p = index(mask, '@')))
+	{
+		*p = '\0';
+		if (match(mask, cptr->username))
+			return 1;
+		mask = p + 1;
+	}
+	if (!(p = index(mask, '/')))
+		goto badmask;
+	*p = '\0';
+	
+	m = atoi(p + 1);
+#ifndef	INET6
+	if (m < 0 || m > 32)
+		goto badmask;
+	lmask = htonl((u_long)0xffffffffL << (32 - m));
+	addr.s_addr = inetaddr(mask);
+	return ((addr.s_addr ^ cptr->ip.s_addr) & lmask) ? 1 : 0;
 #else
-        return ((cptr->ip.s_addr & lmask) == baseip) ? 0 : 1;
+	if (m < 0 || m > 128)
+		goto badmask;
+
+	inet_pton(AF_INET6, mask, (void *)addr.s6_addr);
+
+	j = m & 0x1F;	/* mumber not mutliple of 32 bits */
+	m >>= 5;	/* number of 32 bits */
+
+	if (m && memcmp((void *)(addr.s6_addr), 
+		(void *)(cptr->ip.s6_addr), m << 2))
+		return 1;
+
+	if (j)
+	{
+		lmask = htonl((u_long)0xffffffffL << (32 - j));
+		if ((((u_int32_t *)(addr.s6_addr))[m] ^
+			((u_int32_t *)(cptr->ip.s6_addr))[m]) & lmask)
+			return 1;
+	}
+
+	return 0;
 #endif
+badmask:
+	sendto_flag(SCH_ERROR, "Ignoring bad mask: %s", mask);
+	return -1;
 }
 
 /*
@@ -302,10 +341,10 @@ aClient *cptr;
 		if (ConfMaxHLocal(aconf) > 0 || ConfMaxUHLocal(aconf) > 0) {
 			Reg     aClient *acptr;
 			Reg     int     i;
-			int	sz = sizeof(cptr->ip);
 
 			for (i = highest_fd; i >= 0; i--)
 				if ((acptr = local[i]) && (cptr != acptr) &&
+				    !IsListening(acptr) &&
 				    !bcmp((char *)&cptr->ip,(char *)&acptr->ip,
 					  sizeof(cptr->ip)))
 				    {
@@ -711,6 +750,13 @@ int	sig;
 			free_conf(tmp2);
 	    	    }
 
+	tmp = &kconf;
+	while ((tmp2 = *tmp))
+	    {
+		*tmp = tmp2->next;
+		free_conf(tmp2);
+	    }
+
 	/*
 	 * We don't delete the class table, rather mark all entries
 	 * for deletion. The table is cleaned up by check_class. - avalon
@@ -737,7 +783,7 @@ int	sig;
 				free_conf(tmp2);
 		    }
 #ifdef CACHED_MOTD
-	read_motd(MPATH);
+	read_motd(IRCDMOTD_PATH);
 #endif
 	rehashed = 1;
 	return ret;
@@ -777,7 +823,7 @@ int	openconf()
 		 * goes out with report_error.  Could be dangerous,
 		 * two servers running with the same fd's >:-) -avalon
 		 */
-		(void)execlp("m4", "m4", "ircd.m4", configfile, 0);
+		(void)execlp("m4", "m4", IRCDM4_PATH, configfile, 0);
 		report_error("Error executing m4 %s:%s", &me);
 		_exit(-1);
 	default :
@@ -789,6 +835,69 @@ int	openconf()
 #endif
 }
 
+/*
+** char *ipv6_convert(char *orig)
+** converts the original ip address to an standard form
+** returns a pointer to a string.
+*/
+
+#ifdef	INET6
+char	*ipv6_convert(orig)
+char	*orig;
+{
+	char	*s, *t, *buf = NULL;
+	int	i, j;
+	int	len = 1;	/* for the '\0' in case of no @ */
+	struct	in6_addr addr;
+	char	dummy[MYDUMMY_SIZE];
+
+	if ((s = strchr(orig, '@')))
+	    {
+		*s = '\0';
+		len = strlen(orig) + 2;	/* +2 for '@' and '\0' */
+		buf = (char *)MyMalloc(len);
+		(void *)strcpy(buf, orig);
+		buf[len - 2] = '@';
+		buf[len - 1] = '\0'; 
+		*s = '@';
+		orig = s + 1;
+	    }
+
+	if ((s = strchr(orig, '/')))
+	    {
+		*s = '\0';
+		s++;
+	    }
+
+	i = inet_pton(AF_INET6, orig, addr.s6_addr);
+
+	if (i > 0)
+	    {
+		t = inetntop(AF_INET6, addr.s6_addr, dummy, MYDUMMY_SIZE);
+	    }
+	
+	j = len - 1;
+	if (!((i > 0) && t))
+		t = orig;
+
+	len += strlen(t);
+	buf = (char *)MyRealloc(buf, len);
+	strcpy(buf + j, t);
+
+	if (s)
+	    {
+		*(s-1) = '/'; /* put the '/' back, not sure it's needed tho */ 
+		j = len;
+		len += strlen(s) + 1;
+		buf = (char *)MyRealloc(buf, len);
+		buf[j - 1] = '/';
+		strcpy(buf + j, s);
+	    }
+
+	return buf;
+}
+#endif
+
 /*
 ** initconf() 
 **    Read configuration file.
@@ -985,7 +1094,22 @@ int	opt;
 		    {
 			if ((tmp = getfield(NULL)) == NULL)
 				break;
+#ifdef	INET6
+			if (aconf->status & 
+				(CONF_CONNECT_SERVER|CONF_ZCONNECT_SERVER
+				|CONF_CLIENT|CONF_RCLIENT|CONF_KILL
+				|CONF_OTHERKILL|CONF_NOCONNECT_SERVER
+				|CONF_OPERATOR|CONF_LOCOP|CONF_LISTEN_PORT
+# ifdef R_LINES
+				|CONF_RESTRICT
+# endif
+				|CONF_SERVICE))
+				aconf->host = ipv6_convert(tmp);
+			else
+				DupString(aconf->host, tmp);
+#else
 			DupString(aconf->host, tmp);
+#endif
 			if ((tmp = getfield(NULL)) == NULL)
 				break;
 			DupString(aconf->passwd, tmp);
@@ -994,7 +1118,10 @@ int	opt;
 			DupString(aconf->name, tmp);
 			if ((tmp = getfield(NULL)) == NULL)
 				break;
-			aconf->port = atoi(tmp);
+			aconf->port = 0;
+			if (sscanf(tmp, "0x%x", &aconf->port) != 1 ||
+			    aconf->port == 0)
+				aconf->port = atoi(tmp);
 			if (aconf->status == CONF_CONNECT_SERVER)
 				DupString(tmp2, tmp);
 			if (aconf->status == CONF_ZCONNECT_SERVER)
@@ -1149,8 +1276,16 @@ int	opt;
 		      aconf->name, aconf->port,
 		      aconf->class ? ConfClass(aconf) : 0));
 
-		aconf->next = conf;
-		conf = aconf;
+		if (aconf->status & (CONF_KILL|CONF_OTHERKILL))
+		    {
+			aconf->next = kconf;
+			kconf = aconf;
+		    }
+		else
+		    {
+			aconf->next = conf;
+			conf = aconf;
+		    }
 		aconf = NULL;
 	    }
 	if (aconf)
@@ -1197,24 +1332,30 @@ Reg	aConfItem	*aconf;
 	ln.value.aconf = aconf;
 	ln.flags = ASYNC_CONF;
 
-	if (isdigit(*s))
 #ifdef INET6
-		if(!inet_pton(AF_INET6, s, aconf->ipnum.s6_addr))
-			bcopy(minus_one, aconf->ipnum.s6_addr, IN6ADDRSZ);
+	if(inet_pton(AF_INET6, s, aconf->ipnum.s6_addr))
+		;
 #else
+	if (isdigit(*s))
 		aconf->ipnum.s_addr = inetaddr(s);
 #endif
 	else if ((hp = gethost_byname(s, &ln)))
 		bcopy(hp->h_addr, (char *)&(aconf->ipnum),
 			sizeof(struct IN_ADDR));
+#ifdef	INET6
+	else
+	{
+		bcopy(minus_one, aconf->ipnum.s6_addr, IN6ADDRSZ);
+		goto badlookup;
+	}
 
-#ifdef INET6
-	if (AND16(aconf->ipnum.s6_addr) == 255)
 #else
 	if (aconf->ipnum.s_addr == -1)
-#endif
 		goto badlookup;
+#endif
+
 	return 0;
+
 badlookup:
 #ifdef INET6
 	if (AND16(aconf->ipnum.s6_addr) == 255)
@@ -1258,12 +1399,12 @@ char	**comment;
 
 	*reply = '\0';
 
-	for (tmp = conf; tmp; tmp = tmp->next)
+	for (tmp = kconf; tmp; tmp = tmp->next)
 	    {
 		if (!doall && (BadPtr(tmp->passwd) || !isdigit(*tmp->passwd)))
 			continue;
 		if (!(tmp->status & (CONF_KILL | CONF_OTHERKILL)))
-			continue;
+			continue; /* should never happen with kconf */
 		if (!tmp->host || !tmp->name)
 			continue;
 		if (tmp->status == CONF_KILL)
@@ -1357,11 +1498,16 @@ char	*name, *key;
 int	stat;
 {
 	aConfItem *tmp;
+	int l;
 
+	if (index(key, '/') == NULL)
+		return 0;
+	l = ((char *)index(key, '/') - key) + 1;
 	for (tmp = conf; tmp; tmp = tmp->next)
  		if ((tmp->status == stat) && tmp->passwd && tmp->name &&
- 		    (match(tmp->name, name) == 0) &&
-		    (match(tmp->passwd, key) == 0))
+		    (strncasecmp(key, tmp->passwd, l) == 0) &&
+		    (match(tmp->name, name) == 0) &&
+		    (strpbrk(key + l, tmp->passwd + l)))
 			break;
  	return (tmp ? -1 : 0);
 }
@@ -1574,9 +1720,9 @@ int	class, fd;
 					aconf->name, aconf->port);
 				strcat(rpl, "\r\n");
 #ifdef INET6
-				sendto(class, rpl, strlen(rpl), 0, 0, 0);
+				sendto(fd, rpl, strlen(rpl), 0, 0, 0);
 #else
-				send(class, rpl, strlen(rpl), 0);
+				send(fd, rpl, strlen(rpl), 0);
 #endif
 				return;
 			    }
