@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: s_err.c,v 1.21 1999/01/20 01:28:47 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: s_err.c,v 1.24.2.5 2001/02/10 19:11:06 q Exp $";
 #endif
 
 #include "os.h"
@@ -79,7 +79,7 @@ static	Numeric	numeric_errors[] = {
 		{ 0, (char *)NULL },
 		{ 0, (char *)NULL },
 /* 431 */	{ ERR_NONICKNAMEGIVEN, ":No nickname given" },
-/* 432 */	{ ERR_ERRONEUSNICKNAME, "%s :Erroneous Nickname" },
+/* 432 */	{ ERR_ERRONEOUSNICKNAME, "%s :Erroneous Nickname" },
 /* 433 */	{ ERR_NICKNAMEINUSE, "%s :Nickname is already in use." },
 /* 434 */	{ ERR_SERVICENAMEINUSE, (char *)NULL },
 /* 435 */	{ ERR_SERVICECONFUSED, (char *)NULL },
@@ -181,17 +181,17 @@ static	Numeric	numeric_replies[] = {
 /* 312 */	{ RPL_WHOISSERVER, "%s %s :%s" },
 /* 313 */	{ RPL_WHOISOPERATOR, "%s :is an IRC Operator" },
 /* 314 */	{ RPL_WHOWASUSER, "%s %s %s * :%s" },
-/* 315 */	{ RPL_ENDOFWHO, "%s :End of /WHO list." },
+/* 315 */	{ RPL_ENDOFWHO, "%s :End of WHO list." },
 /* 316 */	{ RPL_WHOISCHANOP, (char *)NULL },
 /* 317 */	{ RPL_WHOISIDLE, "%s %ld :seconds idle" },
-/* 318 */	{ RPL_ENDOFWHOIS, "%s :End of /WHOIS list." },
+/* 318 */	{ RPL_ENDOFWHOIS, "%s :End of WHOIS list." },
 /* 319 */	{ RPL_WHOISCHANNELS, "%s :%s" },
 		{ 0, (char *)NULL },
 /* 321 */	{ RPL_LISTSTART, "Channel :Users  Name" },
 /* 322 */	{ RPL_LIST, "%s %d :%s" },
-/* 323 */	{ RPL_LISTEND, ":End of /LIST" },
+/* 323 */	{ RPL_LISTEND, ":End of LIST" },
 /* 324 */	{ RPL_CHANNELMODEIS, "%s %s %s" },
-/* 325 */	{ RPL_UNIQOPIS, "%s %s %s" },
+/* 325 */	{ RPL_UNIQOPIS, "%s %s" },
 		{ 0, (char *)NULL },
 		{ 0, (char *)NULL },
 		{ 0, (char *)NULL },
@@ -231,8 +231,8 @@ static	Numeric	numeric_replies[] = {
 /* 362 */	{ RPL_CLOSING, "%s :Closed. Status = %d" },
 /* 363 */	{ RPL_CLOSEEND, "%d: Connections Closed" },
 /* 364 */	{ RPL_LINKS, "%s %s :%d %s" },
-/* 365 */	{ RPL_ENDOFLINKS, "%s :End of /LINKS list." },
-/* 366 */	{ RPL_ENDOFNAMES, "%s :End of /NAMES list." },
+/* 365 */	{ RPL_ENDOFLINKS, "%s :End of LINKS list." },
+/* 366 */	{ RPL_ENDOFNAMES, "%s :End of NAMES list." },
 /* 367 */	{ RPL_BANLIST, "%s %s" },
 /* 368 */	{ RPL_ENDOFBANLIST, "%s :End of Channel Ban List" },
 /* 369 */	{ RPL_ENDOFWHOWAS, "%s :End of WHOWAS" },
@@ -240,9 +240,9 @@ static	Numeric	numeric_replies[] = {
 /* 371 */	{ RPL_INFO, ":%s" },
 /* 372 */	{ RPL_MOTD, ":- %s" },
 /* 373 */	{ RPL_INFOSTART, ":Server INFO" },
-/* 374 */	{ RPL_ENDOFINFO, ":End of /INFO list." },
+/* 374 */	{ RPL_ENDOFINFO, ":End of INFO list." },
 /* 375 */	{ RPL_MOTDSTART, ":- %s Message of the Day - " },
-/* 376 */	{ RPL_ENDOFMOTD, ":End of /MOTD command." },
+/* 376 */	{ RPL_ENDOFMOTD, ":End of MOTD command." },
 		{ 0, (char *)NULL },
 		{ 0, (char *)NULL },
 		{ 0, (char *)NULL },
@@ -280,7 +280,7 @@ static	Numeric	numeric_replies[] = {
 /* 204 */	{ RPL_TRACEOPERATOR, "Oper %d %s" },
 /* 205 */	{ RPL_TRACEUSER, "User %d %s" },
 /* 206 */	{ RPL_TRACESERVER, "Serv %d %dS %dC %s %s!%s@%s V%X%s" },
-/* 207 */	{ RPL_TRACESERVICE, "Service %d %s %d %d" },
+/* 207 */	{ RPL_TRACESERVICE, "Service %d %s 0x%X 0x%X" },
 /* 208 */	{ RPL_TRACENEWTYPE, "<newtype> 0 %s" },
 /* 209 */	{ RPL_TRACECLASS, "Class %d %d" },
 /* 210 */	{ RPL_TRACERECONNECT, "Retry. %d %s" },
@@ -292,7 +292,7 @@ static	Numeric	numeric_replies[] = {
 /* 216 */	{ RPL_STATSKLINE, "%c %s %s %s %d %d" },
 /* 217 */	{ RPL_STATSQLINE, "%c %s %s %s %d %d" },
 /* 218 */	{ RPL_STATSYLINE, "%c %d %d %d %d %ld %d.%d %d.%d" },
-/* 219 */	{ RPL_ENDOFSTATS, "%c :End of /STATS report" },
+/* 219 */	{ RPL_ENDOFSTATS, "%c :End of STATS report" },
 		{ 0, (char *)NULL },
 /* 221 */	{ RPL_UMODEIS, "%s" },
 		{ 0, (char *)NULL },
@@ -329,7 +329,7 @@ static	Numeric	numeric_replies[] = {
 /* 252 */	{ RPL_LUSEROP, "%d :operators online" },
 /* 253 */	{ RPL_LUSERUNKNOWN, "%d :unknown connections" },
 /* 254 */	{ RPL_LUSERCHANNELS, "%d :channels formed" },
-/* 255 */	{ RPL_LUSERME, ":I have %d clients, %d services and %d servers" },
+/* 255 */	{ RPL_LUSERME, ":I have %d users, %d services and %d servers" },
 /* 256 */	{ RPL_ADMINME, ":Administrative info about %s" },
 /* 257 */	{ RPL_ADMINLOC1, ":%s" },
 /* 258 */	{ RPL_ADMINLOC2, ":%s" },
@@ -354,8 +354,8 @@ char	*to;
 	num -= numeric_errors[0].num_val;
 	if (num < 0 || num > ERR_USERSDONTMATCH)
 		SPRINTF(numbuff,
-			":%%s %d %%s :INTERNAL ERROR: BAD NUMERIC! %d",
-			numeric, num);
+			":%s %d %s :INTERNAL ERROR: BAD NUMERIC! %d",
+			ME, numeric, to, num);
 	else
 	    {
 		nptr = &numeric_errors[num];
@@ -364,8 +364,8 @@ char	*to;
 			numeric, num, nptr, nptr->num_val, nptr->num_form));
 		if (!nptr->num_form || !nptr->num_val)
 			SPRINTF(numbuff,
-				":%%s %d %%s :NO ERROR FOR NUMERIC ERROR %d",
-				numeric, num);
+				":%s %d %s :NO ERROR FOR NUMERIC ERROR %d",
+				ME, numeric, to, num);
 		else
 			(void)prepbuf(numbuff, ME, to, nptr->num_val,
 				      nptr->num_form);
@@ -381,7 +381,7 @@ char	*to;
 	Reg	Numeric	*nptr;
 	Reg	int	num = numeric;
 
-	if (num > 5)
+	if (num > 100)
 		num -= (num > 300) ? 300 : 100;
 
 	if (BadPtr(to))		/* for unregistered clients */
@@ -389,8 +389,8 @@ char	*to;
 
 	if (num < 0 || num > 200)
 		SPRINTF(numbuff,
-			":%%s %d %%s :INTERNAL REPLY ERROR: BAD NUMERIC! %d",
-			numeric, num);
+			":%s %d %s :INTERNAL REPLY ERROR: BAD NUMERIC! %d",
+			ME, numeric, to, num);
 	else
 	    {
 		if (numeric > 99)
@@ -402,8 +402,8 @@ char	*to;
 			numeric, num, nptr, nptr->num_val, nptr->num_form));
 		if (!nptr->num_form || !nptr->num_val)
 			SPRINTF(numbuff,
-				":%%s %d %%s :NO REPLY FOR NUMERIC ERROR %d",
-				numeric, num);
+				":%s %d %s :NO REPLY FOR NUMERIC ERROR %d",
+				ME, numeric, to, num);
 		else
 			(void)prepbuf(numbuff, ME, to, nptr->num_val,
 				      nptr->num_form);
