@@ -22,7 +22,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: s_misc.c,v 1.26 1999/01/13 02:14:36 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: s_misc.c,v 1.30.2.8 2004/05/09 20:20:33 chopin Exp $";
 #endif
 
 #include "os.h"
@@ -64,13 +64,22 @@ time_t	clock;
 	gm = &gmbuf;
 	lt = localtime(&clock);
 
-	if (lt->tm_yday == gm->tm_yday)
-		minswest = (gm->tm_hour - lt->tm_hour) * 60 +
-			   (gm->tm_min - lt->tm_min);
-	else if (lt->tm_yday > gm->tm_yday)
-		minswest = (gm->tm_hour - (lt->tm_hour + 24)) * 60;
-	else
-		minswest = ((lt->tm_hour + 24) - gm->tm_hour) * 60;
+	minswest = (gm->tm_hour - lt->tm_hour) * 60 
+		    + (gm->tm_min - lt->tm_min);	
+	if (lt->tm_yday != gm->tm_yday)
+	    {
+		if ((lt->tm_yday > gm->tm_yday 
+		    && lt->tm_year == gm->tm_year) 
+		    || (lt->tm_yday < gm->tm_yday 
+		    && lt->tm_year != gm->tm_year)) 
+		    {
+			minswest -= 24 * 60;
+		    }
+		else
+		    {
+			minswest += 24 * 60;
+		    }
+	    }
 
 	plus = (minswest > 0) ? '-' : '+';
 	if (minswest < 0)
@@ -312,6 +321,10 @@ char	*name;
 			acptr->flags |= FLAGS_HIDDEN;
 			j++;
 		}
+		else
+		{
+			acptr->flags &= ~FLAGS_HIDDEN;
+		}
 	}
 	return j;
 }
@@ -350,10 +363,6 @@ char	*comment;	/* Reason for the exit */
 	Reg	aClient	*acptr;
 	Reg	aClient	*next;
 	Reg	aServer *asptr;
-	Reg	aService *asvptr;
-#if defined(FNAME_USERLOG) || defined(USE_SYSLOG) || defined(USE_SERVICES)
-	time_t	on_for;
-#endif
 	char	comment1[HOSTLEN + HOSTLEN + 2];
 	int	flags = 0;
 
@@ -372,34 +381,17 @@ char	*comment;	/* Reason for the exit */
     || (defined(USE_SYSLOG) && (defined(SYSLOG_USERS) || defined(SYSLOG_CONN)))
 		if (IsPerson(sptr))
 		    {
-			/* It's ugly, it's simple, it's not so important */
-			on_for = timeofday - sptr->firsttime + 1;
-# if defined(USE_SYSLOG) && defined(SYSLOG_USERS)
-			syslog(LOG_NOTICE,
-			       "%s (%3d:%02d:%02d): %s@%s [%s] %c\n",
-			       myctime(sptr->firsttime),
-			       on_for / 3600, (on_for % 3600)/60,
-			       on_for % 60,
-			       sptr->user->username, sptr->user->host,
-			       sptr->auth, sptr->exitc);
-# endif
-# if defined(FNAME_USERLOG) || defined(USE_SERVICES)
-			sendto_flog(sptr, NULL, on_for, sptr->user->username,
+# if defined(FNAME_USERLOG) || defined(USE_SERVICES) || \
+	(defined(USE_SYSLOG) && defined(SYSLOG_USERS))
+			sendto_flog(sptr, NULL, sptr->user->username,
 				    sptr->user->host);
 # endif
 		    }
 		else if (sptr->exitc != EXITC_REF && sptr->exitc != EXITC_AREF)
 		    {
-# if defined(USE_SYSLOG) && defined(SYSLOG_CONN)
-			syslog(LOG_NOTICE, 
-			       "%s ( Unknown ): <none>@%s [%s] %c\n",
-			       myctime(sptr->firsttime),
-			       (IsUnixSocket(sptr)) ? me.sockhost :
-			       ((sptr->hostp) ? sptr->hostp->h_name :
-				sptr->sockhost), sptr->auth, sptr->exitc);
-# endif
-# if defined(FNAME_CONNLOG) || defined(USE_SERVICES)
-			sendto_flog(sptr, " Unknown ", 0, "<none>", 
+# if defined(FNAME_CONNLOG) || defined(USE_SERVICES) || \
+	(defined(USE_SYSLOG) && defined(SYSLOG_CONN))
+			sendto_flog(sptr, " Unknown ", "<none>", 
 				    (IsUnixSocket(sptr)) ? me.sockhost :
 				    ((sptr->hostp) ? sptr->hostp->h_name :
 				     sptr->sockhost));
@@ -427,7 +419,7 @@ char	*comment;	/* Reason for the exit */
 
 		      if (sptr->auth != sptr->username)
 			  {
-			    istat.is_authmem -= sizeof(sptr->auth);
+			    istat.is_authmem -= strlen(sptr->auth) + 1;
 			    istat.is_auth -= 1;
 			    MyFree(sptr->auth);
 			    sptr->auth = sptr->username;
@@ -470,91 +462,32 @@ char	*comment;	/* Reason for the exit */
 			(void)strcat(comment1, sptr->name);
 
 			/* This will quit all the *users*, without checking the
-			** whole list of clients. I like it, unfortunately,
-			** there's more below..
+			** whole list of clients.
 			*/
 			for (asptr = svrtop; asptr; asptr = (aServer *)next)
 			    {
 				next = (aClient *)asptr->nexts;
-				if ((asptr->userlist == NULL)
-				    || (asptr->bcptr == NULL)
-				    || (asptr->bcptr->from != sptr
-					&& asptr->bcptr != sptr))
+				if ((asptr->bcptr == NULL) ||
+				    (asptr->bcptr->from != sptr
+				     && asptr->bcptr != sptr))
 					continue;
 				/*
-			        ** This version doesn't need QUITs to be
+				** This version doesn't need QUITs to be
 				** propagaged unless the remote server is
 				** hidden (by a hostmask)
 				*/
+				flags = FLAGS_SPLIT;
 				if (mark_blind_servers(NULL,
 						       asptr->bcptr->name))
-					flags |= FLAGS_SPLIT | FLAGS_HIDDEN;
-				else
-					flags |= FLAGS_SPLIT;
-				do
+					flags |= FLAGS_HIDDEN;
+				while (GotDependantClient(asptr->bcptr))
 				    {
-					acptr = asptr->userlist->bcptr;
+					acptr = asptr->bcptr->prev;
 					acptr->flags |= flags;
 					exit_one_client(NULL, acptr, &me,
 							comment1);
 				    }
-				while (asptr->userlist);
 			    }
-			/* Here is more..
-			** I don't know what use this can have for now, but
-			** the above breaks the old code, which I just pasted
-			** below.
-			** Nothing should happen here, unless there is another
-			** remote entity than "user" (and servers) depending
-			** on remote servers: services,it would be much smarter
-			** to only check services instead of wasting CPU - krys
-			*/
-#if 0
-			for (acptr = client; acptr; acptr = next)
-			    {
-				next = acptr->next;
-				if (!IsServer(acptr) && acptr->from == sptr)
-				    {
-					/*
-					** Let's see if things are ever wrong.
-					** Second step would be: finally change
-					** this loop to go through the services
-					** list. -krys
-					*/
-					if (!IsService(acptr))
-						sendto_flag(SCH_ERROR,
-			    "lists seem corrupted: %s %#x %#x %#x (%s)",
-							    acptr->name,
-							    acptr->user,
-							    acptr->serv,
-							    acptr->service,
-							    sptr->name);
-					exit_one_client(NULL, acptr, &me,
-							comment1);
-				    }
-			    }
-#else
-			/*
-			** I'm now trying to put more comments in this function
-			** than code.  The above could be re-enabled some time
-			** for curiosity/debugging.
-			** I've only heard of one instance when the above
-			** notice showed up, and it was due to a bug, now
-			** fixed.
-			** With 2.9 combined NICK protocol, it should never
-			** happen, as a NICK is immediately associated to a
-			** server.
-			** Avalon made me do it. ;) -krys
-			*/
-			for (asvptr = svctop; asvptr; asvptr =(aService *)next)
-			    {
-				next = (aClient *)asvptr->nexts;
-				if ((acptr = asvptr->bcptr) && 
-				    acptr->from == sptr)
-					exit_one_client(NULL, acptr, &me,
-                                                        comment1);
-			    }
-#endif
 			/*
 			** Second SQUIT all servers behind this link
 			*/
@@ -573,16 +506,15 @@ char	*comment;	/* Reason for the exit */
 		    } /* If (IsServer(sptr)) */
 	    } /* if (MyConnect(sptr) || (sptr->flags & FLAGS_HELD)) */
 
- 	if (IsServer(sptr) && sptr->serv->userlist)
+ 	if (IsServer(sptr) && GotDependantClient(sptr))
  	{
  		/*
 		** generate QUITs locally when receiving a SQUIT
 		** check for hostmasking.
  		*/
+ 		flags = FLAGS_SPLIT;
  		if (mark_blind_servers(cptr, sptr->name))
- 			flags = FLAGS_SPLIT | FLAGS_HIDDEN;
- 		else
- 			flags = FLAGS_SPLIT;
+ 			flags |= FLAGS_HIDDEN;
 
 		if (IsServer(from))
 			/* this is a guess */
@@ -593,13 +525,12 @@ char	*comment;	/* Reason for the exit */
  		(void)strcat(comment1, " ");
  		(void)strcat(comment1, sptr->name);
 
- 		do
+		while (GotDependantClient(sptr))
  		{
- 			acptr = sptr->serv->userlist->bcptr;
+ 			acptr = sptr->prev;
  			acptr->flags |= flags;
 			exit_one_client(cptr, acptr, &me, comment1);
  		}
- 		while (sptr->serv->userlist);
  	}
  	
 	/*
@@ -644,6 +575,14 @@ char	*comment;	/* Reason for the exit */
 	if (IsServer(sptr) && (cptr == sptr))
 		sendto_flag(SCH_SERVER, "Sending SQUIT %s (%s)",
 			    cptr->name, comment);
+	/* non-local server splitted and it matches one of our C-lines,
+	** and we have temporarily disabled AC -- restore it! --B. */
+	if (IsServer(sptr) && (cptr != sptr) &&
+		nextconnect == 0 && find_conf_name(sptr->name,
+		(CONF_CONNECT_SERVER|CONF_ZCONNECT_SERVER)))
+	{
+		nextconnect = timeofday + HANGONRETRYDELAY;
+	}
 	
 	exit_one_client(cptr, sptr, from, (*comment1) ? comment1 : comment);
 	return cptr == sptr ? FLUSH_BUFFER : 0;
@@ -795,6 +734,9 @@ char	*comment;
 #endif
 						 is_chan_op(sptr, lp->value.chptr))
 						lp->value.chptr->history = timeofday + DELAYCHASETIMELIMIT;
+				if (IsAnonymous(lp->value.chptr) &&
+				    !IsQuiet(lp->value.chptr))
+					sendto_channel_butserv(lp->value.chptr, sptr, ":%s PART %s :None", sptr->name, lp->value.chptr->chname);
 				remove_user_from_channel(sptr,lp->value.chptr);
 			    }
 
@@ -808,7 +750,7 @@ char	*comment;
 			add_history(sptr, (sptr->flags & FLAGS_QUIT) ? 
 				    &me : NULL);
 #else
-			add_history(sptr, NULL);
+			add_history(sptr, (sptr == cptr) ? &me : NULL);
 #endif
 			off_history(sptr);
 		    }
@@ -889,7 +831,7 @@ void	initstats()
 {
 	bzero((char *)&istat, sizeof(istat));
 	istat.is_serv = 1;
-	istat.is_remc = 1;	/* don't ask me why, I forgot. */
+	istat.is_localc = 1;	/* &me */
 	bzero((char *)&ircst, sizeof(ircst));
 }
 
@@ -950,35 +892,35 @@ char	*name;
 			sp->is_ni++;
 	    }
 
-	sendto_one(cptr, ":%s %d %s :accepts %u refused %u",
+	sendto_one(cptr, ":%s %d %s :accepts %lu refused %lu",
 		   ME, RPL_STATSDEBUG, name, sp->is_ac, sp->is_ref);
-	sendto_one(cptr, ":%s %d %s :unknown: commands %u prefixes %u",
+	sendto_one(cptr, ":%s %d %s :unknown: commands %lu prefixes %lu",
 		   ME, RPL_STATSDEBUG, name, sp->is_unco, sp->is_unpf);
-	sendto_one(cptr, ":%s %d %s :nick collisions %u unknown closes %u",
+	sendto_one(cptr, ":%s %d %s :nick collisions %lu unknown closes %lu",
 		   ME, RPL_STATSDEBUG, name, sp->is_kill, sp->is_ni);
-	sendto_one(cptr, ":%s %d %s :wrong direction %u empty %u",
+	sendto_one(cptr, ":%s %d %s :wrong direction %lu empty %lu",
 		   ME, RPL_STATSDEBUG, name, sp->is_wrdi, sp->is_empt);
-	sendto_one(cptr, ":%s %d %s :users without servers %u ghosts N/A",
+	sendto_one(cptr, ":%s %d %s :users without servers %lu ghosts N/A",
 		   ME, RPL_STATSDEBUG, name, sp->is_nosrv);
-	sendto_one(cptr, ":%s %d %s :numerics seen %u mode fakes %u",
+	sendto_one(cptr, ":%s %d %s :numerics seen %lu mode fakes %lu",
 		   ME, RPL_STATSDEBUG, name, sp->is_num, sp->is_fake);
-	sendto_one(cptr, ":%s %d %s :auth: successes %u fails %u",
+	sendto_one(cptr, ":%s %d %s :auth: successes %lu fails %lu",
 		   ME, RPL_STATSDEBUG, name, sp->is_asuc, sp->is_abad);
-	sendto_one(cptr,":%s %d %s :local connections %u udp packets %u",
+	sendto_one(cptr,":%s %d %s :local connections %lu udp packets %lu",
 		   ME, RPL_STATSDEBUG, name, sp->is_loc, sp->is_udpok);
-	sendto_one(cptr,":%s %d %s :udp errors %u udp dropped %u",
+	sendto_one(cptr,":%s %d %s :udp errors %lu udp dropped %lu",
 		   ME, RPL_STATSDEBUG, name, sp->is_udperr, sp->is_udpdrop);
 	sendto_one(cptr,
-   ":%s %d %s :link checks %u passed %u 15s/%u 30s dropped %uSq/%uYg/%uFl",
+   ":%s %d %s :link checks %lu passed %lu 15s/%lu 30s dropped %luSq/%luYg/%luFl",
 		   ME, RPL_STATSDEBUG, name, sp->is_ckl, sp->is_cklq,
 		   sp->is_cklok, sp->is_cklQ, sp->is_ckly, sp->is_cklno);
 	if (sp->is_wwcnt)
-		sendto_one(cptr, ":%s %d %s :whowas turnover %u/%u/%u [%u]",
+		sendto_one(cptr, ":%s %d %s :whowas turnover %lu/%lu/%lu [%lu]",
 			   ME, RPL_STATSDEBUG, name, sp->is_wwmt,
 			   (u_int) (sp->is_wwt / sp->is_wwcnt), sp->is_wwMt,
 			   KILLCHASETIMELIMIT);
 	if (sp->is_lkcnt)
-		sendto_one(cptr, ":%s %d %s :ndelay turnover %u/%u/%u [%u]",
+		sendto_one(cptr, ":%s %d %s :ndelay turnover %lu/%lu/%lu [%lu]",
 			   ME, RPL_STATSDEBUG, name, sp->is_lkmt,
 			   (u_int) (sp->is_lkt / sp->is_lkcnt), sp->is_lkMt,
 			   DELAYCHASETIMELIMIT);
@@ -987,15 +929,15 @@ char	*name;
 		   (bootopt & BOOT_STRICTPROT) ? 1 : 0);
 	sendto_one(cptr, ":%s %d %s :Client - Server",
 		   ME, RPL_STATSDEBUG, name);
-	sendto_one(cptr, ":%s %d %s :connected %u %u",
+	sendto_one(cptr, ":%s %d %s :connected %lu %lu",
 		   ME, RPL_STATSDEBUG, name, sp->is_cl, sp->is_sv);
-	sendto_one(cptr, ":%s %d %s :bytes sent %u.%uK %u.%uK",
+	sendto_one(cptr, ":%s %d %s :bytes sent %lu.%luK %lu.%luK",
 		   ME, RPL_STATSDEBUG, name,
 		   sp->is_cks, sp->is_cbs, sp->is_sks, sp->is_sbs);
-	sendto_one(cptr, ":%s %d %s :bytes recv %u.%uK %u.%uK",
+	sendto_one(cptr, ":%s %d %s :bytes recv %lu.%luK %lu.%luK",
 		   ME, RPL_STATSDEBUG, name,
 		   sp->is_ckr, sp->is_cbr, sp->is_skr, sp->is_sbr);
-	sendto_one(cptr, ":%s %d %s :time connected %u %u",
+	sendto_one(cptr, ":%s %d %s :time connected %lu %lu",
 		   ME, RPL_STATSDEBUG, name, sp->is_cti, sp->is_sti);
 #if defined(USE_IAUTH)
 	report_iauth_stats(cptr, name);
@@ -1009,7 +951,7 @@ struct tm	motd_tm;
 void read_motd(filename)
 char *filename;
 {
-	int fd;
+	int fd, len;
 	register aMotd *temp, *last;
 	struct stat Sb;
 	char line[80];
@@ -1031,12 +973,14 @@ char *filename;
 	motd_tm = *localtime(&Sb.st_mtime);
 	(void)dgets(-1, NULL, 0); /* make sure buffer is at empty pos */
 	last = NULL;
-	while (dgets(fd, line, sizeof(line)-1) > 0)
+	while ((len=dgets(fd, line, sizeof(line)-1)) > 0)
 	    {
 		if ((tmp = strchr(line, '\n')) != NULL)
 			*tmp = (char) 0;
-		if ((tmp = strchr(line, '\r')) != NULL)
+		else if ((tmp = strchr(line, '\r')) != NULL)
 			*tmp = (char) 0;
+		else
+			line[len] = '\0';
 		temp = (aMotd *)MyMalloc(sizeof(aMotd));
 		if (!temp)
 			outofmemory();
