@@ -22,7 +22,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: s_serv.c,v 1.53 1999/01/28 23:50:17 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: s_serv.c,v 1.65.2.13 2004/03/01 18:00:33 chopin Exp $";
 #endif
 
 #include "os.h"
@@ -34,6 +34,7 @@ static  char rcsid[] = "@(#)$Id: s_serv.c,v 1.53 1999/01/28 23:50:17 kalt Exp $"
 static	char	buf[BUFSIZE];
 
 static	int	check_link __P((aClient *));
+static	void	report_listeners __P((aClient *, char *));
 
 /*
 ** m_functions execute protocol messages on this server:
@@ -122,7 +123,8 @@ char	*parv[];
 	Reg	aConfItem *aconf;
 	char	*server;
 	Reg	aClient	*acptr;
-	char	*comment = (parc > 2 && parv[2]) ? parv[2] : cptr->name;
+	char	*comment = (parc > 2 && parv[2]) ? parv[2] : "no reason";
+	static char	comment2[TOPICLEN+1];
 
 	if (parc > 1)
 	    {
@@ -209,6 +211,57 @@ char	*parv[];
 		sendto_one(sptr, err_str(ERR_NOPRIVILEGES, parv[0]));
 		return 1;
 	    }
+	if (MyPerson(sptr))
+	{
+		int	k=TOPICLEN-strlen(sptr->name)-7;
+
+		/* Shorten original comment, should it be too long. */
+		if (strlen(comment) > k)
+		{
+			comment[k] = '\0';
+		}
+		/* As we change comment, we have to copy, who knows what
+		** parv[2] can overwrite. */
+		comment2[0] = '\0';
+		sprintf(comment2, "%s (by %s)", comment, sptr->name);
+		comment = comment2;
+	}
+	if (!MyConnect(acptr) && (cptr != acptr->from))
+	    {
+		/*
+		** The following is an awful kludge, but I don't see any other
+		** way to change the pre 2.10.3 behaviour.  I'm probably going
+		** to regret it.. -kalt
+		*/
+		if ((acptr->from->serv->version & SV_OLDSQUIT) == 0)
+		    {
+			/* better server: just propagate upstream */
+			sendto_one(acptr->from, ":%s SQUIT %s :%s", parv[0],
+				   acptr->name, comment);
+			sendto_flag(SCH_SERVER,
+				    "Forwarding SQUIT %s from %s (%s)",
+				    acptr->name, parv[0], comment);
+			sendto_flag(SCH_DEBUG,
+				    "Forwarding SQUIT %s to %s from %s (%s)",
+				    acptr->name, acptr->from->name, 
+				    parv[0], comment);
+			return 1;
+		    }
+		/*
+		** ack, bad server encountered!
+		** must send back to other good servers which were trying to
+		** do the right thing, and fake the yet to come SQUIT which
+		** will never be received from the bad servers.
+		*/
+		if (IsServer(cptr) && 
+		    (cptr->serv->version & SV_OLDSQUIT) == 0)
+		    {
+			sendto_one(cptr, ":%s SQUIT %s :%s (Bounced for %s)",
+				   ME, acptr->name, comment, parv[0]);
+			sendto_flag(SCH_DEBUG, "Bouncing SQUIT %s back to %s",
+				    acptr->name, acptr->from->name);
+		    }
+	    }
 	/*
 	**  Notify all opers, if my local link is remotely squitted
 	*/
@@ -236,6 +289,13 @@ char	*parv[];
 	sendto_flag(SCH_SERVER, "Received SQUIT %s from %s (%s)",
 		    acptr->name, parv[0], comment);
 
+	if (MyConnect(acptr) && 
+	    IsServer(cptr) && (cptr->serv->version & SV_OLDSQUIT) == 0)
+	    {
+		sendto_one(cptr, ":%s SQUIT %s :%s", ME, acptr->name, comment);
+		sendto_flag(SCH_DEBUG, "Issuing additionnal SQUIT %s for %s",
+			    acptr->name, acptr->from->name);
+	    }
 	return exit_client(cptr, acptr, sptr, comment);
     }
 
@@ -274,27 +334,18 @@ aClient	*cptr;
 	else
 		id = "";
 
-	if (!strncmp(cptr->info, "021", 3) ||
-	    !strncmp(cptr->info, "020999", 6))
+	if (!strncmp(cptr->info, "021", 3))
 		cptr->hopcount = SV_29|SV_NJOIN|SV_NMODE|SV_NCHAN; /* SV_2_10*/
 	else if (!strncmp(cptr->info, "0209", 4))
-	    {
-		cptr->hopcount = SV_29;	/* 2.9+ protocol */
-		if (!(cptr->info[4] == '0' &&
-		      (cptr->info[5] == '0' || cptr->info[5] == '1' ||
-		       cptr->info[5] == '2' || cptr->info[5] == '3' ||
-		       cptr->info[5] == '4' ||
-		       (cptr->info[5] == '5' && cptr->info[6] == '0' &&
-			cptr->info[7] == '0'))))
-			/*
-			** the NJOIN command appeared on 2.9.5
-			** Unfortunately, m_njoin() has a fatal bug in 2.9.5
-			*/
-			cptr->hopcount |= SV_NJOIN;
-	    }
+		cptr->hopcount = SV_29|SV_OLDSQUIT;	/* 2.9+ protocol */
 	else
 		cptr->hopcount = SV_OLD; /* uhuh */
 
+	if (!strcmp("IRC", id) && !strncmp(cptr->info, "02100", 5) &&
+	    atoi(cptr->info+5) < 20600)
+		/* before 2.10.3a6 ( 2.10.3a5 is just broken ) */
+		cptr->hopcount |= SV_OLDSQUIT;
+
 	/* Check version number/mask from conf */
 	sprintf(buf, "%s/%s", id, cptr->info);
 	if (find_two_masks(cptr->name, buf, CONF_VER))
@@ -672,27 +723,22 @@ Reg	aClient	*cptr;
 	    }
 
 #ifdef CRYPT_LINK_PASSWORD
-	/* use first two chars of the password they send in as salt */
+	/* pass whole aconf->passwd as salt, let crypt() deal with it */
 
-	/* passwd may be NULL. Head it off at the pass... */
 	if (*cptr->passwd)
 	    {
-		char    salt[3];
 		extern  char *crypt();
 
-		/* Determine if MD5 or DES */
-                if (strncmp(aconf->passwd, "$1$", 3))
+		encr = crypt(cptr->passwd, aconf->passwd);
+		if (encr == NULL)
 		    {
-			salt[0] = aconf->passwd[0];
-			salt[1] = aconf->passwd[1];
+			ircstp->is_ref++;
+			sendto_one(cptr, "ERROR :No Access (crypt failed) %s",
+			  	inpath);
+			sendto_flag(SCH_ERROR,
+			    	"Access denied (crypt failed) %s", inpath);
+			return exit_client(cptr, cptr, &me, "Bad Password");
 		    }
-		else
-		    {
-			salt[0] = aconf->passwd[3];
-			salt[1] = aconf->passwd[4];
-		    }
-		salt[2] = '\0';
-		encr = crypt(cptr->passwd, salt);
 	    }
 	else
 		encr = "";
@@ -838,8 +884,6 @@ Reg	aClient	*cptr;
 #endif
 	sendto_flag(SCH_SERVER, "Sending SERVER %s (%d %s)", cptr->name,
 		    1, cptr->info);
-	sendto_flag(SCH_DEBUG, "Burst to %s: %X%s", inpath,
-		    cptr->hopcount, (cptr->flags & FLAGS_ZIP) ? "z" : "");
 	/*
 	** Old sendto_serv_but_one() call removed because we now
 	** need to send different names to different servers
@@ -902,9 +946,6 @@ Reg	aClient	*cptr;
 				   acptr->hopcount+1, stok, acptr->info);
 	    }
 
-	sendto_flag(SCH_DEBUG, "SERVER phase complete: %u",
-		    (int)DBufLength(&cptr->sendQ));
-
 	for (acptr = &me; acptr; acptr = acptr->prev)
 	    {
 		/* acptr->from == acptr for acptr == cptr */
@@ -917,12 +958,6 @@ Reg	aClient	*cptr;
 			** These are only true when *BOTH* NICK and USER have
 			** been received. -avalon
 			*/
-			if (acptr->user->servp->userlist == NULL)
-				sendto_flag(SCH_ERROR,
-			    "ERROR: USER:%s without SERVER:%s(%d) (to %s)",
-					    acptr->name, acptr->user->server,
-					    acptr->user->servp->tok,
-					    cptr->name);
 			if (*mlname == '*' &&
 			    match(mlname, acptr->user->server) == 0)
 				stok = me.serv->tok;
@@ -953,8 +988,7 @@ Reg	aClient	*cptr;
 		/* the previous if does NOT catch all services.. ! */
 	    }
 
-	sendto_flag(SCH_DEBUG, "client phase complete: %u",
-		    (int)DBufLength(&cptr->sendQ));
+	flush_connections(cptr->fd);
 
 	/*
 	** Last, pass all channels modes
@@ -971,9 +1005,6 @@ Reg	aClient	*cptr;
 			    }
 	    }
 
-	sendto_flag(SCH_DEBUG, "NJOIN/mode phase complete: %u",
-		    (int)DBufLength(&cptr->sendQ));
-
 	cptr->flags &= ~FLAGS_CBURST;
 #ifdef	ZIP_LINKS
  	/*
@@ -1093,8 +1124,6 @@ char	*parv[];
 
 	if (parc > 2)
 	    {
-		int	qlen = strlen(parv[2]);
-
 		if (IsServer(cptr) && check_link(cptr) && !IsOper(sptr))
 		    {
 			sendto_one(sptr, rpl_str(RPL_TRYAGAIN, parv[0]),
@@ -1286,7 +1315,8 @@ int	mask;
 	int	*p, port;
 	char	c, *host, *pass, *name;
 	
-	for (tmp = conf; tmp; tmp = tmp->next)
+	for (tmp = (mask & (CONF_KILL|CONF_OTHERKILL)) ? kconf : conf;
+	     tmp; tmp = tmp->next)
 		if (tmp->status & mask)
 		    {
 			for (p = &report_array[0][0]; *p; p += 3)
@@ -1335,7 +1365,7 @@ char	*to;
 				   buf, cp->lseq, cp->lrecvd,
 				   cp->ping / (cp->recvd ? cp->recvd : 1),
 				   tmp->pref);
-			sendto_flag(SCH_DEBUG, "%s: %d", buf, cp->seq);
+			/* sendto_flag(SCH_DEBUG, "%s: %d", buf, cp->seq); */
 		    }
 	return;
 }
@@ -1350,20 +1380,43 @@ char	*parv[];
 	aClient	*acptr;
 	char	stat = parc > 1 ? parv[1][0] : '\0';
 	Reg	int	i;
-	int	doall = 0, wilds = 0;
-	char	*name = NULL, *cm = NULL;
+	int	wilds, doall;
+	char	*name, *cm;
 
-	if (IsServer(cptr) &&
-	    (stat != 'd' && stat != 'p' && stat != 'q' && stat != 's' &&
-	     stat != 'u' && stat != 'v') &&
-	    !(stat == 'o' && IsOper(sptr)))
+	if (IsServer(cptr))
 	    {
+		switch(stat)
+                {
+		/* These stats are available with no penalty for all. */
+		case 'd': case 'D':     /* defines */
+		case 'p':               /* ping stats */
+		case 'P':               /* ports listening */
+		case 'q': case 'Q':     /* Q:lines */
+		case 's': case 'S':     /* services */
+		case 'u': case 'U':     /* uptime */
+		case 'v': case 'V':     /* V:lines */
+		case 'l': case 'L':     /* links (wildcard is dropped later) */
+			break;
+		/* These are available with no penalty for opers. */
+		/* Although I have no idea, why only for opers. --B. */
+		case 'o': case 'O':     /* O:lines */
+		case 'c': case 'C':     /* C:/N: lines */
+		case 'h': case 'H':     /* H:/D: lines */
+		case 'a': case 'A':     /* iauth conf */
+		case 'b': case 'B':     /* B:lines */
+			if (IsOper(sptr))
+			{
+				break;
+			}
+			/* else fallthrough */
+                default:
 		if (check_link(cptr))
 		    {
 			sendto_one(sptr, rpl_str(RPL_TRYAGAIN, parv[0]),
 				   "STATS");
 			return 5;
 		    }
+		}
 	    }
 	if (parc == 3)
 	    {
@@ -1371,31 +1424,17 @@ char	*parv[];
 				2, parc, parv) != HUNTED_ISME)
 			return 5;
 	    }
-	else if (parc == 4)
+	else if (parc >= 3)
 	    {
 		if (hunt_server(cptr, sptr, ":%s STATS %s %s %s",
 				2, parc, parv) != HUNTED_ISME)
 			return 5;
 	    }
 
-	if (parc > 2)
-	    {
-		name = parv[2];
-		if (!mycmp(name, ME))
-			doall = 2;
-		else if (match(name, ME) == 0)
-			doall = 1;
-		if (index(name, '*') || index(name, '?'))
-			wilds = 1;
-		if (parc > 3)
-		    {
-			cm = parv[3];
-			if (!index(cm, '*') && !index(cm, '?'))
-				wilds = 0, doall = 0;
-		    }
-	    }
-	else
-		name = ME;
+	name = (parc > 2) ? parv[2] : ME;
+	cm = (parc > 3) ? parv[3]: name;
+	doall = !match(name, ME) && !match(cm, ME);
+	wilds = index(cm, '*') || index(cm, '?');
 
 	switch (stat)
 	{
@@ -1406,34 +1445,45 @@ char	*parv[];
 		 * are invisible not being visible to 'foreigners' who use
 		 * a wild card based search to list it.
 		 */
-		for (i = 0; i <= highest_fd; i++)
+		if (doall || wilds)
 		    {
-			if (!(acptr = local[i]))
-				continue;
-#if 0
-			if (IsPerson(acptr) && IsInvisible(acptr) &&
-			    (doall || wilds) && !(MyConnect(sptr) &&
-			     IsLocal(sptr) && IsOper(sptr)) &&
-			    !IsAnOper(acptr) && acptr != sptr)
-#endif
-			if (IsPerson(acptr) &&
-			    (doall || wilds) &&
-			    !(MyConnect(sptr) && IsAnOper(sptr)) &&
-			    acptr != sptr)
-				continue;
-			if (!doall && wilds && match(name, acptr->name))
-				continue;
-			if (!(doall || wilds) &&
-			    ((!cm && mycmp(name, acptr->name)) ||
-			     (cm && match(cm, acptr->name))))
-				continue;
-			sendto_one(cptr, Lformat, ME,
-				   RPL_STATSLINKINFO, parv[0],
-				   get_client_name(acptr, isupper(stat)),
-				   (int)DBufLength(&acptr->sendQ),
-				   (int)acptr->sendM, (int)acptr->sendK,
-				   (int)acptr->receiveM, (int)acptr->receiveK,
-				   timeofday - acptr->firsttime);
+			if (IsServer(cptr) && check_link(cptr))
+                        {
+				sendto_one(sptr, rpl_str(RPL_TRYAGAIN, parv[0]),
+				   "STATS");
+                                return 5;
+                        }
+			for (i = 0; i <= highest_fd; i++)
+			    {
+				if (!(acptr = local[i]))
+					continue;
+				if (IsPerson(acptr) && !(MyConnect(sptr) 
+				    && IsAnOper(sptr)) && acptr != sptr)
+					continue;
+				if (wilds && match(cm, acptr->name))
+					continue;
+				sendto_one(cptr, Lformat, ME,
+					RPL_STATSLINKINFO, parv[0],
+					get_client_name(acptr, isupper(stat)),
+					(int)DBufLength(&acptr->sendQ),
+					(int)acptr->sendM, (int)acptr->sendK,
+					(int)acptr->receiveM, 
+					(int)acptr->receiveK,
+					timeofday - acptr->firsttime);
+			    }
+		    }
+		else
+		    {
+			if ((acptr = find_client(cm, NULL)) && MyConnect(acptr))
+				sendto_one(cptr, Lformat, ME,
+					RPL_STATSLINKINFO, parv[0],
+					get_client_name(acptr, isupper(stat)),
+					(int)DBufLength(&acptr->sendQ),
+					(int)acptr->sendM, (int)acptr->sendK,
+					(int)acptr->receiveM,
+					(int)acptr->receiveK,
+					timeofday - acptr->firsttime);
+			
 		    }
 		break;
 #if defined(USE_IAUTH)
@@ -1475,7 +1525,10 @@ char	*parv[];
 	case 'o' : case 'O' : /* O (and o) lines */
 		report_configured_links(cptr, parv[0], CONF_OPS);
 		break;
-	case 'p' : case 'P' : /* ircd ping stats */
+	case 'P' : /* ports listening */
+		report_listeners(sptr, parv[0]);
+		break;
+	case 'p' : /* ircd ping stats */
 		report_ping(sptr, parv[0]);
 		break;
 	case 'Q' : case 'q' : /* Q lines */
@@ -1816,7 +1869,7 @@ char	*parv[];
 	if (!aconf)
 	    {
 	      sendto_one(sptr,
-			 "NOTICE %s :Connect: Host %s not listed in irc.conf",
+			 "NOTICE %s :Connect: Host %s not listed in ircd.conf",
 			 parv[0], parv[1]);
 	      return 0;
 	    }
@@ -1980,7 +2033,8 @@ aClient	*cptr, *sptr;
 int	parc;
 char	*parv[];
 {
-	sendto_one(sptr, rpl_str(RPL_REHASHING, parv[0]), configfile);
+	sendto_one(sptr, rpl_str(RPL_REHASHING, parv[0]),
+		   mybasename(configfile));
 	sendto_flag(SCH_NOTICE,
 		    "%s is rehashing Server config file", parv[0]);
 #ifdef USE_SYSLOG
@@ -2005,6 +2059,7 @@ char	*parv[];
 	char	killer[HOSTLEN * 2 + USERLEN + 5];
 
 	strcpy(killer, get_client_name(sptr, TRUE));
+	SPRINTF(buf, "RESTART by %s", get_client_name(sptr, TRUE));
 	for (i = 0; i <= highest_fd; i++)
 	    {
 		if (!(acptr = local[i]))
@@ -2026,7 +2081,6 @@ char	*parv[];
 	    }
 	flush_connections(me.fd);
 
-	SPRINTF(buf, "RESTART by %s", get_client_name(sptr, TRUE));
 	restart(buf);
 	/*NOT REACHED*/
 	return 0;
@@ -2130,7 +2184,7 @@ char	*parv[];
 		case STAT_ME:
 			break;
 		case STAT_UNKNOWN:
-			if (IsAnOper(sptr) || MyClient(sptr))
+			if (IsAnOper(sptr) || (MyPerson(sptr) && SendWallops(sptr)))
 				sendto_one(sptr,
 					   rpl_str(RPL_TRACEUNKNOWN, parv[0]),
 					   class, name);
@@ -2139,15 +2193,6 @@ char	*parv[];
 			/* Only opers see users if there is a wildcard
 			 * but anyone can see all the opers.
 			 */
-/*
-			if (IsOper(sptr)  &&
-			    (MyClient(sptr) || !(dow && IsInvisible(acptr)))
-			    || !dow || IsAnOper(acptr))
-			    {
-			if (IsOper(sptr) && !(dow || IsInvisible(acptr)) ||
-			    (IsOper(sptr) && IsLocal(sptr)) ||
-			    !dow || IsAnOper(acptr))
-*/
 			if (IsAnOper(acptr))
 				sendto_one(sptr,
 					   rpl_str(RPL_TRACEOPERATOR, parv[0]),
@@ -2156,17 +2201,6 @@ char	*parv[];
 				sendto_one(sptr,
 					   rpl_str(RPL_TRACEUSER, parv[0]),
 					   class, name);
-/*
-			    {
-				if (IsAnOper(acptr))
-					sendto_one(sptr,
-						   rpl_str(RPL_TRACEOPERATOR,
-						   parv[0]), class, name);
-				else
-					sendto_one(sptr, rpl_str(RPL_TRACEUSER,
-						   parv[0]), class, name);
-			    }
-*/
 			break;
 		case STAT_SERVER:
 			if (acptr->serv->user)
@@ -2269,7 +2303,7 @@ char	*parv[];
 	 * 3 seconds. -avalon (curtesy of wumpus)
 	 */
 	(void)alarm(3);
-	fd = open(MPATH, O_RDONLY);
+	fd = open(IRCDMOTD_PATH, O_RDONLY);
 	(void)alarm(0);
 	if (fd == -1)
 	    {
@@ -2468,3 +2502,19 @@ aClient	*cptr;
     ircstp->is_cklno++;
     return -1;
 }
+
+static void report_listeners(aClient *sptr, char *to)
+{
+	aConfItem *tmp;
+
+	for (tmp = conf; tmp; tmp = tmp->next)
+	{
+		if ((tmp->status & CONF_LISTEN_PORT))
+		{
+			sendto_one(sptr, ":%s %d %s :%s %d %d", ME,
+				RPL_STATSDEFINE, to, BadPtr(tmp->host) ?
+				"*" : tmp->host, tmp->port,
+				tmp->clients);
+		}
+	}
+}
